/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : gui                                                    
** Fichier : gui_interface.h
** Description : gui widgets
------------------------------------------------------------------------------*/
/**
 * \file gui_interface.h
 * \brief Module : gui, Widgets de l'Interface graphique
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef GUI_INTERFACE_H
#define GUI_INTERFACE_H
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <cairo.h>
#include <math.h>
#include "gui_callbacks.h"
#include "gui_support.h"
#include "const_signalisation.h"
#include "const_aiguillages.h"
#include "const_receptivities.h"
#include "const_trains.h"
#include "const_detelages.h"

#define AIGUILLAGES_TXT_CENTRE "centre"
#define	AIGUILLAGES_TXT_GAUCHE "gauche"
#define	AIGUILLAGES_TXT_DROITE "droite"
#define	AIGUILLAGES_TXT_CROISEMENT "croisement"
#define	AIGUILLAGES_TXT_BIFURCATION "bifurcation"

#define DETELAGES_TXT_DET1 "DET1"
#define DETELAGES_TXT_DET2 "DET2"
#define DETELAGES_TXT_DET3 "DET3"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget* TSGUI_mainWindow;

/**
 * \fn GtkWidget* TSGUI_create_mainWindow(void)
 * \brief Creation de la fenetre principale
 * \return GtkWidget fenetre principale
 */
GtkWidget* TSGUI_create_mainWindow(void);

/**
 * \fn void TSGUI_update(int threadSafe)
 * \brief Mise a jour de l'interface graphique
 * \param threadSafe commutateur de thread
 * \return void
 */
void TSGUI_update(int threadSafe);
/**
 * \fn void TSGUI_updateCircuit(void)
 * \brief Mise a jour du circuit
 * \return void
 */
void TSGUI_updateCircuit(void);
/**
 * \fn void TSGUI_updateLog(void)
 * \brief Mise a jour du log
 * \return void
 */
void TSGUI_updateLog(void);
#endif
