/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : gui                                                    
** Fichier : gui_support.h
** Description : gui support
------------------------------------------------------------------------------*/
/**
 * \file gui_support.h
 * \brief Module : gui, Assistance GUI
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef GUI_SUPPORT_H
#define GUI_SUPPORT_H
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gtk/gtk.h>

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  define Q_(String) g_strip_context ((String), gettext (String))
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define Q_(String) g_strip_context ((String), (String))
#  define N_(String) (String)
#endif


/*
 * Public Functions.
 */
/**
 * \fn GtkWidget* lookup_widget(GtkWidget *widget,const gchar *widget_name)
 * \brief This function returns a widget in a component created by Glade.
 * Call it with the toplevel widget in the component (i.e. a window/dialog),
 * or alternatively any widget in the component, and the name of the widget
 * you want returned.
 * \param widget widget de recherche
 * \param widget_name nom du widget recherche
 * \return GtkWidget* pointeur sur widget
 */
GtkWidget* lookup_widget(GtkWidget *widget,const gchar *widget_name);

/**
 * \fn void add_pixmap_directory (const gchar *directory)
 * \brief Use this function to set the directory containing installed pixmaps.
 * \param directory répertoire
 * \return void
 */
void add_pixmap_directory (const gchar *directory);


/*
 * Private Functions.
 */

/**
 * \fn GtkWidget* create_pixmap(GtkWidget *widget,const gchar  *filename)
 * \brief This is used to create the pixmaps used in the interface.
 * \param widget widget de recherche
 * \param file_name nom du pixmap
 * \return GtkWidget* pointeur sur widget
 */
GtkWidget* create_pixmap(GtkWidget *widget,const gchar  *filename);

/**
 * \fn GdkPixbuf* create_pixbuf(const gchar *filename)
 * \brief This is used to create the pixbufs used in the interface.
 * \param file_name nom du pixbuf
 * \return GdkPixbuf* pointeur sur pixbuf
 */
GdkPixbuf* create_pixbuf(const gchar *filename);

/**
 * \fn void glade_set_atk_action_description (AtkAction *action,const gchar *action_name,const gchar *description)
 * \brief This is used to set ATK action descriptions.
 * \param action
 * \param action_name
 * \param description
 * \return GdkPixbuf* pointeur sur pixbuf
 */
void glade_set_atk_action_description (AtkAction *action,const gchar *action_name,const gchar *description);

#endif
