/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel.c                                     
** Description : Noyau                                
------------------------------------------------------------------------------*/
/**
 * \file kernel.c
 * \brief Module : kernel, Noyau
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_kernel_run_set(int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_run);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_run);
	#endif
	TSKERNEL_run=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_run);
	#else
		sem_post(&TSKERNEL_SEM_kernel_run);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_kernel_run_get(void){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_run);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_run);
	#endif
	value=TSKERNEL_run;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_run);
	#else
		sem_post(&TSKERNEL_SEM_kernel_run);
	#endif
	return value;
}

int TSKERNEL_RES_kernel_simul_state_set(int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_simul_state);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_simul_state);
	#endif
	TSKERNEL_simul_state=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_simul_state);
	#else
		sem_post(&TSKERNEL_SEM_kernel_simul_state);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_kernel_simul_state_get(void){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_simul_state);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_simul_state);
	#endif
	value=TSKERNEL_simul_state;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_simul_state);
	#else
		sem_post(&TSKERNEL_SEM_kernel_simul_state);
	#endif
	return value;
}

int TSKERNEL_RES_kernel_simul_run_set(int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_simul_run);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_simul_run);
	#endif
	TSKERNEL_simul_run=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_simul_run);
	#else
		sem_post(&TSKERNEL_SEM_kernel_simul_run);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_kernel_simul_run_get(void){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_kernel_simul_run);
	#else
		sem_wait(&TSKERNEL_SEM_kernel_simul_run);
	#endif
	value=TSKERNEL_simul_run;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_kernel_simul_run);
	#else
		sem_post(&TSKERNEL_SEM_kernel_simul_run);
	#endif
	return value;
}

/*
TSKERNEL
*/
int TSKERNEL_pre_init(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"kernel : init");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_kernel_run=rt_sem_init(TSKERNEL_SEM_kernel_run,1);
		TSKERNEL_SEM_kernel_simul_state=rt_sem_init(TSKERNEL_SEM_kernel_simul_state,1);
		TSKERNEL_SEM_kernel_simul_run=rt_sem_init(TSKERNEL_SEM_kernel_simul_run,1);
	#else
		sem_init(&TSKERNEL_SEM_kernel_run,0,1);
		sem_init(&TSKERNEL_SEM_kernel_simul_state,0,1);
		sem_init(&TSKERNEL_SEM_kernel_simul_run,0,1);
	#endif
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation du circuit
	TSKERNEL_MOD_circuit_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation des receptivites
	TSKERNEL_MOD_receptivities_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation des blocks
	TSKERNEL_MOD_blocks_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation des trains
	TSKERNEL_MOD_trains_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation des aiguillages
	TSKERNEL_MOD_aiguillages_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation des rails de detelages
	TSKERNEL_MOD_detelages_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	//Initialisation de la signalisation
	TSKERNEL_MOD_signalisation_load();
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;
	
	TSKERNEL_RES_kernel_run_set(TSKERNEL_RUN_ON);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_init(void){
	TSKERNEL_pre_init();
	//Initialisation port serie
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) TSKERNEL_MOD_serial_load();
	
	TSKERNEL_RES_kernel_run_set(TSKERNEL_RUN_ON);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_loop(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"kernel : loop begin");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_end(void){
	//Arret du circuit
	TSKERNEL_MOD_circuit_unload();
	//Arret des receptivites
	TSKERNEL_MOD_receptivities_unload();
	//Arret des blocks
	TSKERNEL_MOD_blocks_unload();
	//Arret des trains
	TSKERNEL_MOD_trains_unload();
	//Arret des aiguillages
	TSKERNEL_MOD_aiguillages_unload();
	//Arret des rails de detelages
	TSKERNEL_MOD_detelages_unload();
	//Arret de la signalisation
	TSKERNEL_MOD_signalisation_unload();
	//Arret port serie
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) TSKERNEL_MOD_serial_unload();

	TSKERNEL_RES_kernel_run_set(TSKERNEL_RUN_OFF);
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_kernel_run);
		rt_sem_delete(TSKERNEL_SEM_kernel_simul_state);
		rt_sem_delete(TSKERNEL_SEM_kernel_simul_run);
	#else
		sem_destroy(&TSKERNEL_SEM_kernel_run);
		sem_destroy(&TSKERNEL_SEM_kernel_simul_state);
		sem_destroy(&TSKERNEL_SEM_kernel_simul_run);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"kernel : end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_stop(void){
	TSKERNEL_RES_kernel_run_set(TSKERNEL_RUN_STOP);

	//Stop child process
	system("killall -s 2 trainsim_USR");

	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"kernel : stop");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_runSimul(void){
	TSKERNEL_CMD_simul_on();
	return TSKERNEL_exec();
}

int TSKERNEL_runReal(void){
	TSKERNEL_CMD_simul_off();
	return TSKERNEL_exec();
}

int TSKERNEL_exec(void){
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_simulButton")), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_realButton")), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_stopButton")), TRUE);

	TSKERNEL_RES_kernel_simul_run_set(TSKERNEL_RUN_ON);
	TSKERNEL_init();

	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;

	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"kernel : exec begin");
	//Fork process
	char* args[]={SOCKET_PRG, NULL};
	TS_tasks_exec(args);

	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_STOP) return TS_ERROR_EXIT;

	//Open socket
	char recv_c;
	TSKERNEL_MOD_socket_load();

	//Refresh receptivities
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF){
			TSKERNEL_MOD_receptivities_refreshAll(1);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"kernel : refresh receptivities");

	//Read RECV
	int recv_state=-1;
	char* param0=(char*)malloc(1024*sizeof(char));
	char* param0_1=(char*)malloc(1024*sizeof(char));
	int op_code=-1;
	int param1=-1;
	int param2=-1;
	int param3=-1;
	int ret=0;
	char* query=(char*)malloc(1024*sizeof(char));
	strcpy(query,"");

	//Kernel loop
	while (1 == TSKERNEL_MOD_socket_read(&recv_c)){
		TSGUI_update(TSKERNEL_THREADSAFE_OFF);

		//Parse Query
		char* recv_str=(char*)malloc(5*sizeof(char));
		sprintf(recv_str,"%c",recv_c);
		strcat(query,recv_str);
		free(recv_str);

		if (recv_c==API_QUERY_START_CHAR){//Begin op
			recv_state=1;
			op_code=-1;
			param1=-1;
			param2=-1;
			param3=-1;
		} else if (recv_c==API_QUERY_END_CHAR){//End op
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"kernel : simul recv command : %s",query);
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INTERNAL,message);
			free(message);

			//printf("%s\n",query);
			
			char** querySplitted=str_split(query,API_DELIMITER_STR);
			if (querySplitted[1]!=NULL)
				op_code=atoi(querySplitted[1]);
			if (querySplitted[2]!=NULL)
				param1=atoi(querySplitted[2]);
			if (querySplitted[3]!=NULL)
				param2=atoi(querySplitted[3]);
			if (querySplitted[4]!=NULL)
				param3=atoi(querySplitted[4]);
			
			switch(op_code){
				case API_OPCODE_AIGUILLAGES_SETPOS:
					param0=querySplitted[2];
					param0_1=querySplitted[3];
					ret=TSAPI_aiguillages_setPosition(param0,param0_1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_BLOCKS_MUTEX_P:
					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);
					ret=TSAPI_blocks_mutex_P(param1,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_BLOCKS_MUTEX_V:
					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);
					ret=TSAPI_blocks_mutex_V(param1,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_CIRCUIT_RELACHEMENT:
					ret=TSAPI_circuit_relachement();
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_CIRCUIT_DISJONCTION:
					ret=TSAPI_circuit_disjonction();
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_CIRCUIT_CONJONCTION:
					ret=TSAPI_circuit_conjonction();
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_DETELAGES_GETPOS:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_detelages_getPosition(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_DETELAGES_SETPOS:
					param0=querySplitted[2];
					param2=atoi(querySplitted[3]);
					ret=TSAPI_detelages_setPosition(param0,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_RECEPTIVITIES_GET:
					//Refresh receptivities
					if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF){
							TSKERNEL_MOD_receptivities_refreshAll(1);
					}
					TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"kernel : refresh receptivities");

					TSGUI_update(TSKERNEL_THREADSAFE_OFF);

					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);
					ret=TSAPI_receptivities_get(param1,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_SIGNALISATION_SET:
					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);
					ret=TSAPI_signalisation_setState(param1,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_SIGNALISATION_GET:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_signalisation_getState(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_GETSPEED:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_trains_getSpeed(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_GETPOS:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_trains_getPosition(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_SETSPEED:
					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);
					param3=atoi(querySplitted[4]);
					ret=TSAPI_trains_setSpeed(param1,param2,param3);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_REVERSE:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_trains_reverse(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_CREATE:
					param1=atoi(querySplitted[2]);
					param2=atoi(querySplitted[3]);					
					ret=TSAPI_trains_createItinerary(param1,param2);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
				case API_OPCODE_TRAINS_DELETE:
					param1=atoi(querySplitted[2]);
					ret=TSAPI_trains_deleteItinerary(param1);
					TSKERNEL_MOD_socket_sendAnswer(ret);
				break;
			}
			
			strcpy(query,"");
			recv_state=0;
		}
		TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	}
	//Relachement
	TSKERNEL_MOD_circuit_relachement(); //par securite

	//Close sockets
	TSKERNEL_MOD_socket_unload();

	free(query);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"kernel : exec end");
	TSKERNEL_RES_kernel_simul_run_set(TSKERNEL_RUN_OFF);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_simulButton")), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_realButton")), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_cmd_stopButton")), FALSE);

	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_trace(int codeType,int type,char* message){
	char* txt;
	txt=(char*) malloc(1024*sizeof(char));

	#ifndef TS_NOGUI
		GtkTextIter start;
		GtkTextIter end;
		GtkTextBuffer* TSGUI_logTextView_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (lookup_widget(GTK_WIDGET(TSGUI_mainWindow),"TSGUI_logTextView")));

		gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(TSGUI_logTextView_buffer),&start);
		gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(TSGUI_logTextView_buffer),&end);
	#endif

	if (type<=TSKERNEL_trace_level){
		switch(type){
			case TSKERNEL_TRACE_ERROR:
				switch(codeType){
					case  TSKERNEL_CODE_USER:
						sprintf(txt,"[TSKERNEL_ERROR] %s\n",message);
						#ifdef TSKERNEL_DEBUG_TO_CONSOLE
							printf(txt);
						#endif
						#ifndef TS_NOGUI
							gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(TSGUI_logTextView_buffer), &end,(gchar*)txt, -1, "size_medium","foreground_white","background_red", NULL);
						#else
							printf(txt);
						#endif						 
					break;
					/*case TSKERNEL_CODE_KERNEL_NORT:
						printk("[TSKERNEL_ERREUR] %s\n",message);
					break;
					case TSKERNEL_CODE_KERNEL_RT:
						rt_printk("[TSKERNEL_ERREUR] %s\n",message);
					break;*/
				}
			break;
			case TSKERNEL_TRACE_INFO:
				switch(codeType){
					case  TSKERNEL_CODE_USER:
						sprintf(txt,"[TSKERNEL_INFO ] %s\n",message);
						#ifdef TSKERNEL_DEBUG_TO_CONSOLE
							printf(txt);
						#endif
						#ifndef TS_NOGUI
							gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(TSGUI_logTextView_buffer), &end,(gchar*)txt, -1, "size_medium","foreground_white","background_blue", NULL);
						#else
							printf(txt);
						#endif
					break;
					/*case TSKERNEL_CODE_KERNEL_NORT:
						printk("[TSKERNEL_INFO] %s\n",message);
					break;
					case TSKERNEL_CODE_KERNEL_RT:
						rt_printk("[TSKERNEL_INFO] %s\n",message);
					break;*/
				}
			break;
			case TSKERNEL_TRACE_DEBUG:
				switch(codeType){
					case  TSKERNEL_CODE_USER:
						sprintf(txt,"[TSKERNEL_DEBUG] %s\n",message);
						#ifdef TSKERNEL_DEBUG_TO_CONSOLE
							printf(txt);
						#endif
						#ifndef TS_NOGUI
							gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(TSGUI_logTextView_buffer), &end,(gchar*)txt, -1, "size_medium", NULL);
						#else
							printf(txt);
						#endif
					break;
					/*case TSKERNEL_CODE_KERNEL_NORT:
						printk("[TSKERNEL_DEBUG] %s\n",message);
					break;
					case TSKERNEL_CODE_KERNEL_RT:
						rt_printk("[TSKERNEL_DEBUG] %s\n",message);
					break;*/
				}
			break;
			case TSKERNEL_TRACE_INTERNAL:
				switch(codeType){
					case  TSKERNEL_CODE_USER:
						sprintf(txt,"[TSKERNEL_INTER] %s\n",message);
						#ifdef TSKERNEL_DEBUG_TO_CONSOLE
							printf(txt);
						#endif
						#ifndef TS_NOGUI
							gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(TSGUI_logTextView_buffer), &end,(gchar*)txt, -1, "size_medium", NULL);
						#else
							printf(txt);
						#endif
					break;
					/*case TSKERNEL_CODE_KERNEL_NORT:
						printk("[TSKERNEL_INTER] %s\n",message);
					break;
					case TSKERNEL_CODE_KERNEL_RT:
						rt_printk("[TSKERNEL_INTER] %s\n",message);
					break;*/
				}
			break;
		}
	}
	#ifndef TS_NOGUI
		//Auto-scroll log
		TSGUI_updateLog();
	#endif
	free(txt);
	return TSKERNEL_NORMAL_EXIT;
}

/*
TSKERNEL_CMD
*/
/*_trace*/
int TSKERNEL_CMD_trace_setLevel(int level){
	switch(level){
		case TSKERNEL_TRACE_ERROR:
		case TSKERNEL_TRACE_INFO:
		case TSKERNEL_TRACE_DEBUG:
		case TSKERNEL_TRACE_NO:
			TSKERNEL_trace_level=level;
			return TSKERNEL_NORMAL_EXIT;
		break;
		default:
			return TSKERNEL_INVALID_TRACE_LEVEL;
		break;
	}
}

/*_interface*/
int TSKERNEL_CMD_interface_sendOneNoRelachement(int msg){
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"%c",msg);
	char* trace=(char*)malloc(1024*sizeof(char));
	sprintf(trace,"interface : sendOne(%d)",msg);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,trace);
	free(trace);
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) TSKERNEL_MOD_serial_send(1,message);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_CMD_interface_sendOne(int msg){
	int ret=TSKERNEL_CMD_interface_sendOneNoRelachement(msg);
	ret+=TSKERNEL_MOD_circuit_relachement();
	return ret;
}

int TSKERNEL_CMD_interface_sendTwoNoRelachement(int msg1,int msg2){
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"%c%c",msg1,msg2);
	char* trace=(char*)malloc(1024*sizeof(char));
	sprintf(trace,"interface : sendTwo(%d,%d)",msg1,msg2);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,trace);
	free(trace);
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) TSKERNEL_MOD_serial_send(2,message);
	return TSKERNEL_NORMAL_EXIT;
}
int TSKERNEL_CMD_interface_sendTwo(int msg1,int msg2){
	int ret=TSKERNEL_CMD_interface_sendTwoNoRelachement(msg1,msg2);
	ret+=TSKERNEL_MOD_circuit_relachement();
	return ret;
}

int TSKERNEL_CMD_interface_read(){
	int readData=0;
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) readData=TSKERNEL_MOD_serial_read();
	char* trace=(char*)malloc(1024*sizeof(char));
	sprintf(trace,"interface : read(%d)",readData);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,trace);
	free(trace);
	return readData;
}

/*_simul*/
int TSKERNEL_CMD_simul_on(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"simulation : simul=on");
	TSKERNEL_RES_kernel_simul_state_set(TSKERNEL_SIMUL_ON);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_CMD_simul_off(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"simulation : simul=off");
	TSKERNEL_RES_kernel_simul_state_set(TSKERNEL_SIMUL_OFF);
	return TSKERNEL_NORMAL_EXIT;
}



