/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel.h                                     
** Description : Noyau                                
------------------------------------------------------------------------------*/
/**
 * \file kernel.h
 * \brief Module : kernel, Noyau
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_H
#define KERNEL_H
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include "const_kernel.h"
#include "kernel_mod_aiguillages.h"
#include "kernel_mod_blocks.h"
#include "kernel_mod_circuit.h"
#include "kernel_mod_detelages.h"
#include "kernel_mod_receptivities.h"
#include "kernel_mod_signalisation.h"
#include "kernel_mod_trains.h"
#include "kernel_mod_serial.h"
#include "kernel_mod_socket.h"
#include "gui_interface.h"
#include "const_socket.h"
#include "const_api.h"
#include "string_extra.h"

#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_kernel_run;
	static SEM* TSKERNEL_SEM_kernel_simul_state;
	static SEM* TSKERNEL_SEM_kernel_simul_run;
#else
	static sem_t TSKERNEL_SEM_kernel_run;
	static sem_t TSKERNEL_SEM_kernel_simul_state;
	static sem_t TSKERNEL_SEM_kernel_simul_run;
#endif
/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_kernel_run_set(int value)
 * \brief Change la ressource
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_kernel_run_set(int value);
/**
 * \fn int TSKERNEL_RES_kernel_run_get(void)
 * \brief Renvoie la valeur de la ressource
 * \return int valeur
 */
int TSKERNEL_RES_kernel_run_get(void);
/**
 * \fn int TSKERNEL_RES_kernel_simulState_set(int value)
 * \brief Change la ressource
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_kernel_simulState_set(int value);
/**
 * \fn int TSKERNEL_RES_kernel_simulState_get(void)
 * \brief Renvoie la valeur de la ressource
 * \return int valeur
 */
int TSKERNEL_RES_kernel_simulState_get(void);
/**
 * \fn int TSKERNEL_RES_kernel_simul_run_set(int value)
 * \brief Change la ressource
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_kernel_simul_run_set(int value);
/**
 * \fn int TSKERNEL_RES_kernel_simul_run_get(void)
 * \brief Renvoie la valeur de la ressource
 * \return int valeur
 */
int TSKERNEL_RES_kernel_simul_run_get(void);

/*
TSKERNEL
*/
static int TSKERNEL_run=TSKERNEL_RUN_OFF;
static int TSKERNEL_simul_state=TSKERNEL_SIMUL_ON;
static int TSKERNEL_simul_run=TSKERNEL_RUN_OFF;

static int TSKERNEL_trace_level=TSKERNEL_TRACE_NO;

/**
 * \fn int TSKERNEL_pre_init(void)
 * \brief Pre-initialisation du noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_pre_init(void);
/**
 * \fn int TSKERNEL_init(void)
 * \brief Initialisation du noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_init(void);
/**
 * \fn int TSKERNEL_loop(void)
 * \brief Boucle du noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_loop(void);
/**
 * \fn int TSKERNEL_end(void)
 * \brief Fin du noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_end(void);
/**
 * \fn int TSKERNEL_stop(void)
 * \brief Arret du noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_stop(void);
/**
 * \fn int TSKERNEL_runSimul(void)
 * \brief Lancement de la simulation
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_runSimul(void);
/**
 * \fn int TSKERNEL_runReal(void)
 * \brief Lancement de la commande du circuit
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_runReal(void);
/**
 * \fn int TSKERNEL_exec(void)
 * \brief Execution noyau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_exec(void);
/**
 * \fn int TSKERNEL_trace(int codeType,int type,char* message)
 * \brief Fonction de trace
 * \param codeType type de code
 * \param type type de trace
 * \param message message
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_trace(int codeType,int type,char* message);

/*
TSKERNEL_CMD
*/
/*_trace*/
/**
 * \fn int TSKERNEL_CMD_trace_setLevel(int level)
 * \brief Changer le niveau de trace
 * \param level niveau
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_trace_setLevel(int level);
/*_interface*/
/**
 * \fn int TSKERNEL_CMD_interface_sendOneNoRelachement(int msg)
 * \brief Envoie d'un message de longueur 1 sans relachement
 * \param msg message
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_interface_sendOneNoRelachement(int msg);
/**
 * \fn int TSKERNEL_CMD_interface_sendOneNoRelachement(int msg)
 * \brief Envoie d'un message de longueur 1 avec relachement
 * \param msg message
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_interface_sendOne(int msg);
/**
 * \fn int TSKERNEL_CMD_interface_sendTwoNoRelachement(int msg1,int msg2)
 * \brief Envoie d'un message de longueur 2 sans relachement
 * \param msg1 message 1
 * \param msg2 message 2
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_interface_sendTwoNoRelachement(int msg1,int msg2);
/**
 * \fn int TSKERNEL_CMD_interface_sendTwo(int msg1,int msg2)
 * \brief Envoie d'un message de longueur 2 avec relachement
 * \param msg1 message 1
 * \param msg2 message 2
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_interface_sendTwo(int msg1,int msg2);
/**
 * \fn int TSKERNEL_CMD_interface_read(void)
 * \brief Lecture d'un message
 * \return int messsage
 */
int TSKERNEL_CMD_interface_read(void);
/*_simul*/
/**
 * \fn int TSKERNEL_CMD_simul_on(void)
 * \brief Active l'etat de simulation
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_simul_on(void);
/**
 * \fn int TSKERNEL_CMD_simul_off(void)
 * \brief Desactive l'etat de simulation
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_CMD_simul_off(void);

#endif
