/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_aiguillages.c                                     
** Description : Module noyau aiguillages                                    
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_aiguillages.c
 * \brief Module : kernel, Module noyau aiguillages
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_aiguillages.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_aiguillages_positions_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_aiguillages_positions);
	#else
		sem_wait(&TSKERNEL_SEM_aiguillages_positions);
	#endif
	TSKERNEL_AIGUILLAGES_POSITIONS[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_aiguillages_positions);
	#else
		sem_post(&TSKERNEL_SEM_aiguillages_positions);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_aiguillages_positions_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_aiguillages_positions);
	#else
		sem_wait(&TSKERNEL_SEM_aiguillages_positions);
	#endif
	value=TSKERNEL_AIGUILLAGES_POSITIONS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_aiguillages_positions);
	#else
		sem_post(&TSKERNEL_SEM_aiguillages_positions);
	#endif
	return value;
}

int TSKERNEL_RES_aiguillages_nums_get(int i){
	int value=0;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_aiguillages_nums);
	#else
		sem_wait(&TSKERNEL_SEM_aiguillages_nums);
	#endif
	value=AIGUILLAGES_NUMS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_aiguillages_nums);
	#else
		sem_post(&TSKERNEL_SEM_aiguillages_nums);
	#endif
	return value;
}

/*
TSKERNEL_MOD_aiguillages
*/
/*_load*/
int TSKERNEL_MOD_aiguillages_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"aiguillages : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_aiguillages_nums=rt_sem_init(TSKERNEL_SEM_aiguillages_nums,1);
		TSKERNEL_SEM_aiguillages_positions=rt_sem_init(TSKERNEL_SEM_aiguillages_positions,1);
	#else
		sem_init(&TSKERNEL_SEM_aiguillages_nums,0,1);
		sem_init(&TSKERNEL_SEM_aiguillages_positions,0,1);
	#endif
	int i=0;
	for(i=0;i<AIGUILLAGES_NB_AIGUILLAGES;i++){
		TSKERNEL_MOD_aiguillages_simple_setPosition(TSKERNEL_RES_aiguillages_nums_get(i),AIGUILLAGES_POSITION_1);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"aiguillages : load end");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_unload(void){
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_aiguillages_nums);
		rt_sem_delete(TSKERNEL_SEM_aiguillages_positions);
	#else
		sem_destroy(&TSKERNEL_SEM_aiguillages_nums);
		sem_destroy(&TSKERNEL_SEM_aiguillages_positions);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"aiguillages : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_getOffset(int aiguillage_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_aiguillages_simple_isValidNum(aiguillage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE;
	//Execution
	int i=0;
	for (i=0;i<AIGUILLAGES_NB_AIGUILLAGES;i++){
		if (TSKERNEL_RES_aiguillages_nums_get(i)==aiguillage_num) 
			break;
	}
	return i;
}

int TSKERNEL_MOD_aiguillages_getPosition(int aiguillage_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_aiguillages_simple_isValidNum(aiguillage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE;
	//Execution
	return TSKERNEL_RES_aiguillages_positions_get(TSKERNEL_MOD_aiguillages_getOffset(aiguillage_num));
}

/*_simple*/
int TSKERNEL_MOD_aiguillages_simple_isValidPosition(int pos){
	if (pos != AIGUILLAGES_POSITION_1 && pos != AIGUILLAGES_POSITION_2){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : invalid position %d",pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_simple_isValidNum(int aiguillage_num){
	int i=0;
	for (i=0;i<AIGUILLAGES_NB_AIGUILLAGES;i++){
		if (TSKERNEL_RES_aiguillages_nums_get(i)==aiguillage_num) 
			return TSKERNEL_NORMAL_EXIT;
	}
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"aiguillages : invalid aiguillage %d",aiguillage_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
	free(message);
	return TSKERNEL_INVALID_AIGUILLAGE;
}

int TSKERNEL_MOD_aiguillages_simple_setPosition(int aiguillage_num,int pos){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_aiguillages_simple_isValidPosition(pos)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	if (TSKERNEL_MOD_aiguillages_simple_isValidNum(aiguillage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE;
	//Execution
	int ret=TSKERNEL_CMD_interface_sendTwo(pos,aiguillage_num);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : position changed for aiguillage %d to %d",aiguillage_num,pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_aiguillages_positions_set(TSKERNEL_MOD_aiguillages_getOffset(aiguillage_num),pos);
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

/*_double*/
int TSKERNEL_MOD_aiguillages_double_isValidPosition(int pos1,int pos2){
	if ((TSKERNEL_MOD_aiguillages_simple_isValidPosition(pos1)!=TSKERNEL_NORMAL_EXIT) || (TSKERNEL_MOD_aiguillages_simple_isValidPosition(pos2)!=TSKERNEL_NORMAL_EXIT) || (pos1==AIGUILLAGES_POSITION_2 && pos2==AIGUILLAGES_POSITION_1)){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : invalid positions peer %d %d",pos1,pos2);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_double_isValidNum(int aiguillage_num1,int aiguillage_num2){
	int ok=0;
	int i=0;
	for (i=0;i<AIGUILLAGES_NB_AIGUILLAGES;i++){
		int tmp_num=TSKERNEL_RES_aiguillages_nums_get(i);
		if (tmp_num==aiguillage_num1 || tmp_num==aiguillage_num2){
			ok++;
			if (ok==2) break;
		}
	}
	if (ok!=2){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : invalid aiguillage [%d,%d]",aiguillage_num1,aiguillage_num2);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_AIGUILLAGE;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_double_setPosition(int aiguillage_num1,int pos1,int aiguillage_num2,int pos2){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_aiguillages_double_isValidPosition(pos1,pos2)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	
	if (TSKERNEL_MOD_aiguillages_double_isValidNum(aiguillage_num1,aiguillage_num2)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE;
	//Execution
	int ret= TSKERNEL_CMD_interface_sendTwo(pos1,aiguillage_num1);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : position changed for aiguillage %d to %d",aiguillage_num1,pos1);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_aiguillages_positions_set(TSKERNEL_MOD_aiguillages_getOffset(aiguillage_num1),pos1);
	}
	ret+=TSKERNEL_CMD_interface_sendTwo(pos2,aiguillage_num2);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : position changed for aiguillage %d to %d",aiguillage_num2,pos2);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_aiguillages_positions_set(TSKERNEL_MOD_aiguillages_getOffset(aiguillage_num2),pos2);
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

/*_croisement*/
int TSKERNEL_MOD_aiguillages_croisement_isValidPosition(int pos){
	if (pos != AIGUILLAGES_CROISEMENT_POSITION_CROISEMENT && pos != AIGUILLAGES_CROISEMENT_POSITION_BIFURCATION){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : invalid position %d",pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_croisement_iSValidNum(int aiguillage_num){
	if (aiguillage_num!=AIGUILLAGES_CROISEMENT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : invalid aiguillage %d",aiguillage_num);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_AIGUILLAGE;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_aiguillages_croisement_setPosition(int aiguillage_num,int pos){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_aiguillages_croisement_isValidPosition(pos)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE_POSITION;
	if (TSKERNEL_MOD_aiguillages_croisement_iSValidNum(aiguillage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_AIGUILLAGE;
	//Execution
	int ret=TSKERNEL_CMD_interface_sendTwo(pos,aiguillage_num);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"aiguillages : position changed for crossroad %d to %d",aiguillage_num,pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_aiguillages_positions_set(TSKERNEL_MOD_aiguillages_getOffset(aiguillage_num),pos);
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

