/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_aiguillages.h                                    
** Description : Module noyau aiguillages                                    
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_aiguillages.h
 * \brief Module : kernel, Module noyau aiguillages
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_AIGUILLAGES_H
#define KERNEL_MOD_AIGUILLAGES_H
#include "const_aiguillages.h"
#include "const_kernel.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_aiguillages_nums;
	static SEM* TSKERNEL_SEM_aiguillages_positions;
#else
	static sem_t TSKERNEL_SEM_aiguillages_nums;
	static sem_t TSKERNEL_SEM_aiguillages_positions;
#endif

/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_aiguillages_positions_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_aiguillages_positions_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_aiguillages_positions_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_aiguillages_positions_get(int i);
/**
 * \fn int TSKERNEL_RES_aiguillages_nums_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_aiguillages_nums_get(int i);

/*
TSKERNEL_MOD_aiguillages
*/
static int TSKERNEL_AIGUILLAGES_POSITIONS[AIGUILLAGES_NB_AIGUILLAGES]={
AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,
AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,
AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1,
AIGUILLAGES_POSITION_1,AIGUILLAGES_POSITION_1};

/**
 * \fn int TSKERNEL_MOD_aiguillages_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_load(void);
/**
 * \fn int TSKERNEL_MOD_aiguillages_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_unload(void);

/**
 * \fn int TSKERNEL_MOD_aiguillages_getOffset(int aiguillage_num)
 * \brief Renvoie l'offset de l'aiguillage
 * \param aiguillage_num numero de l'aiguillage
 * \return int offset
 */
int TSKERNEL_MOD_aiguillages_getOffset(int aiguillage_num);
/**
 * \fn int TSKERNEL_MOD_aiguillages_getPosition(int aiguillage_num)
 * \brief Renvoie la position de l'aiguillage
 * \param aiguillage_num numero de l'aiguillage
 * \return int position
 */
int TSKERNEL_MOD_aiguillages_getPosition(int aiguillage_num);
/*_simple*/
/**
 * \fn int TSKERNEL_MOD_aiguillages_simple_isValidPosition(int pos)
 * \brief Teste la position
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_simple_isValidPosition(int pos);
/**
 * \fn int TSKERNEL_MOD_aiguillages_simple_isValidNum(int aiguillage_num)
 * \brief Teste le numero d'aiguillage
 * \param aiguillage_num numero d'aiguillage
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_simple_isValidNum(int aiguillage_num);
/**
 * \fn int TSKERNEL_MOD_aiguillages_simple_setPosition(int aiguillage_num,int pos)
 * \brief Change la position d'un aiguillage
 * \param aiguillage_num numero d'aiguillage
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_simple_setPosition(int aiguillage_num,int pos);

/*_double*/
/**
 * \fn int TSKERNEL_MOD_aiguillages_double_isValidPosition(int pos1,int pos2)
 * \brief Teste la position double
 * \param pos1 position
 * \param pos2 position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_double_isValidPosition(int pos1,int pos2);
/**
 * \fn int TSKERNEL_MOD_aiguillages_double_isValidNum(int aiguillage_num1,int aiguillage_num2)
 * \brief Teste les numeros d'un aiguillage double
 * \param aiguillage_num1 numero d'aiguillage
 * \param aiguillage_num2 numero d'aiguillage
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_double_isValidNum(int aiguillage_num1,int aiguillage_num2);
/**
 * \fn int TSKERNEL_MOD_aiguillages_double_setPosition(int aiguillage_num1,int pos1,int aiguillage_num2,int pos2)
 * \brief Change la position d'un aiguillage double
 * \param aiguillage_num1 numero d'aiguillage
 * \param pos1 position
 * \param aiguillage_num2 numero d'aiguillage
 * \param pos2 position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_double_setPosition(int aiguillage_num1,int pos1,int aiguillage_num2,int pos2);

/*_croisement*/
/**
 * \fn int TSKERNEL_MOD_aiguillages_croisement_isValidPosition(int pos)
 * \brief Teste la position
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_croisement_isValidPosition(int pos);
/**
 * \fn int TSKERNEL_MOD_aiguillages_croisement_iSValidNum(int aiguillage_num)
 * \brief Teste le numero de croisement
 * \param aiguillage_num numero de croisement
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_croisement_iSValidNum(int aiguillage_num);
/**
 * \fn int TSKERNEL_MOD_aiguillages_croisement_setPosition(int aiguillage_num,int pos)
 * \brief Change la position d'un croisement
 * \param aiguillage_num numero de croisement
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_aiguillages_croisement_setPosition(int aiguillage_num,int pos);

#endif
