/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_blocks.c                                     
** Description : Module noyau blocs
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_blocks.c
 * \brief Module : kernel, Module noyau blocs
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_blocks.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_blocks_trains_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_blocks_trains);
	#else
		sem_wait(&TSKERNEL_SEM_blocks_trains);
	#endif
	TSKERNEL_BLOCKS_TRAINS[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_blocks_trains);
	#else
		sem_post(&TSKERNEL_SEM_blocks_trains);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_blocks_trains_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_blocks_trains);
	#else
		sem_wait(&TSKERNEL_SEM_blocks_trains);
	#endif
	value=TSKERNEL_BLOCKS_TRAINS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_blocks_trains);
	#else
		sem_post(&TSKERNEL_SEM_blocks_trains);
	#endif
	return value;
}

/*
TSKERNEL_MOD_blocks
*/
int TSKERNEL_MOD_blocks_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"blocks : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_blocks_trains=rt_sem_init(TSKERNEL_SEM_blocks_trains,1);
	#else
		sem_init(&TSKERNEL_SEM_blocks_trains,0,1);
	#endif

	int i=0;
/*	for(i=0;i<BLOCKS_NB_BLOCKS;i++){
		#ifdef TS_RTAI
			TSKERNEL_SEM_blocks_sem[i]=rt_sem_init(TSKERNEL_SEM_blocks_sem[i],1);
		#else
			sem_init(&TSKERNEL_SEM_blocks_sem[i],0,1);
		#endif
	}*/
	//Initialisation
	for(i=0;i<BLOCKS_NB_BLOCKS;i++){
		TSKERNEL_RES_blocks_trains_set(i,TRAINS_LOCOMOTIVE_NOT_ASSIGNED);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"blocks : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_blocks_unload(void){
	//Destruction des semaphores
	/*int i=0;
	for(i=0;i<BLOCKS_NB_BLOCKS;i++){
		#ifdef TS_RTAI
			rt_sem_delete(TSKERNEL_SEM_blocks_sem[i]);
		#else
			sem_destroy(&TSKERNEL_SEM_blocks_sem[i]);
		#endif
	}
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_blocks_trains);
	#else
		sem_destroy(&TSKERNEL_SEM_blocks_trains);
	#endif*/
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"blocks : unload");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_blocks_isValidBlock(int block_num){
	if (block_num<BLOCKS_NOT_ASSIGNED || block_num>BLOCKS_G2){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"blocks : invalid block %d",block_num);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_BLOCK;
	}
	else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_blocks_isValidActiveBlock(int block_num){
	if(TSKERNEL_MOD_blocks_isValidBlock(block_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_BLOCK;
	/*
	int i=0;
	for (i=0;i<TRAINS_NB_LOCOMOTIVES;i++){
		if (TSKERNEL_RES_trains_positions_get(i)==block_num) return TSKERNEL_NORMAL_EXIT;
	}
	*/
	if (TSKERNEL_RES_blocks_trains_get(block_num)!=TRAINS_LOCOMOTIVE_NOT_ASSIGNED) return TSKERNEL_NORMAL_EXIT;
	return TSKERNEL_INACTIVE_BLOCK;
}

int TSKERNEL_MOD_blocks_getActiveTrain(int block_num){
	if(TSKERNEL_MOD_blocks_isValidBlock(block_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_BLOCK;
	return TSKERNEL_RES_blocks_trains_get(block_num);
}

int TSKERNEL_blocks_mutex_P(int block_num,int train_num){
	if(TSKERNEL_MOD_blocks_isValidBlock(block_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_BLOCK;
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	/*#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_blocks_sem[block_num]);
	#else
		sem_wait(&TSKERNEL_SEM_blocks_sem[block_num]);
	#endif*/
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"blocks : P(block %d) for train %d",block_num,train_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);
	TSKERNEL_MOD_trains_setPosition(train_num,block_num);
	TSKERNEL_RES_blocks_trains_set(block_num,train_num);
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_blocks_mutex_V(int block_num,int train_num){
	if(TSKERNEL_MOD_blocks_isValidBlock(block_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_BLOCK;
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	/*#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_blocks_sem[block_num]);
	#else
		sem_post(&TSKERNEL_SEM_blocks_sem[block_num]);
	#endif*/
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"blocks : V(block %d) for train %d",block_num,train_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);
	TSKERNEL_RES_blocks_trains_set(block_num,TRAINS_LOCOMOTIVE_NOT_ASSIGNED);
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_NORMAL_EXIT;
}

