/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_blocks.h                                    
** Description : Module noyau blocs
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_blocks.h
 * \brief Module : kernel, Module noyau blocs
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_BLOCKS_H
#define KERNEL_MOD_BLOCKS_H
#include "const_blocks.h"
#include "const_trains.h"
#include "const_kernel.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	/*static SEM* TSKERNEL_SEM_blocks_sem[BLOCKS_NB_BLOCKS];//Attention indice 0 non utilisé car block 0 = hors voie et plusieurs trains peuvent être hors voie*/
	static SEM* TSKERNEL_SEM_blocks_trains;
#else
	/*static sem_t TSKERNEL_SEM_blocks_sem[BLOCKS_NB_BLOCKS];//Attention indice 0 non utilisé car block 0 = hors voie et plusieurs trains peuvent être hors voie*/
	static sem_t TSKERNEL_SEM_blocks_trains;
#endif

/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_blocks_trains_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_blocks_trains_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_blocks_trains_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_blocks_trains_get(int i);

/*
TSKERNEL_MOD_blocks
*/
static int TSKERNEL_BLOCKS_TRAINS[BLOCKS_NB_BLOCKS]={TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,
TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED,TRAINS_LOCOMOTIVE_NOT_ASSIGNED};

/**
 * \fn int TSKERNEL_MOD_blocks_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_blocks_load(void);
/**
 * \fn int TSKERNEL_MOD_blocks_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_blocks_unload(void);

/**
 * \fn int TSKERNEL_MOD_blocks_isValidBlock(int block_num)
 * \brief Teste le numero du block
 * \param block_num numero de block
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_blocks_isValidBlock(int block_num);
/**
 * \fn int TSKERNEL_MOD_blocks_isValidActiveBlock(int block_num)
 * \brief Teste si le numero du block est actif
 * \param block_num numero de block
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_blocks_isValidActiveBlock(int block_num);
/**
 * \fn int TSKERNEL_MOD_blocks_getActiveTrain(int block_num)
 * \brief Renvoie le numéro du train actif
 * \param block_num numero de block
 * \return int numero de train
 */
int TSKERNEL_MOD_blocks_getActiveTrain(int block_num);
/**
 * \fn int TSKERNEL_blocks_mutex_P(int block_num,int train_num)
 * \brief Prise de mutex sur un block
 * \param block_num numero de block
 * \param train_num numero de train
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_blocks_mutex_P(int block_num,int train_num);
/**
 * \fn int TSKERNEL_blocks_mutex_V(int block_num,int train_num)
 * \brief Liberation de mutex sur un block
 * \param block_num numero de block
 * \param train_num numero de train
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_blocks_mutex_V(int block_num,int train_num);

#endif
