/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_detelages.c                                     
** Description : Module noyau detelage                               
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_detelages.c
 * \brief Module : kernel, Module noyau detelages
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_detelages.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_detelages_positions_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_detelages_positions);
	#else
		sem_wait(&TSKERNEL_SEM_detelages_positions);
	#endif
	TSKERNEL_DETELAGES_RAILS_POSITIONS[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_detelages_positions);
	#else
		sem_post(&TSKERNEL_SEM_detelages_positions);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_detelages_positions_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_detelages_positions);
	#else
		sem_wait(&TSKERNEL_SEM_detelages_positions);
	#endif
	value=TSKERNEL_DETELAGES_RAILS_POSITIONS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_detelages_positions);
	#else
		sem_post(&TSKERNEL_SEM_detelages_positions);
	#endif
	return value;
}

int TSKERNEL_RES_detelages_nums_get(int i){
	int value=0;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_detelages_nums);
	#else
		sem_wait(&TSKERNEL_SEM_detelages_nums);
	#endif
	value=DETELAGES_NUMS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_detelages_nums);
	#else
		sem_post(&TSKERNEL_SEM_detelages_nums);
	#endif
	return value;
}

/*
TSKERNEL_MOD_detelages
*/
int TSKERNEL_MOD_detelages_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"detelages : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_detelages_nums=rt_sem_init(TSKERNEL_SEM_detelages_nums,1);
		TSKERNEL_SEM_detelages_positions=rt_sem_init(TSKERNEL_SEM_detelages_positions,1);
	#else
		sem_init(&TSKERNEL_SEM_detelages_nums,0,1);
		sem_init(&TSKERNEL_SEM_detelages_positions,0,1);
	#endif
	int i=0;
	for(i=0;i<DETELAGES_NB_RAILS;i++){
		TSKERNEL_MOD_detelages_setPosition(TSKERNEL_RES_detelages_nums_get(i),DETELAGES_POSITION_OFF);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"detelages : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_detelages_unload(void){
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_detelages_nums);
		rt_sem_delete(TSKERNEL_SEM_detelages_positions);
	#else
		sem_destroy(&TSKERNEL_SEM_detelages_nums);
		sem_destroy(&TSKERNEL_SEM_detelages_positions);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"detelages : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_detelages_getOffset(int detelage_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_detelages_isValidNum(detelage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_DETELAGE;
	//Execution
	int i=0;
	for (i=0;i<DETELAGES_NB_RAILS;i++){
		if (TSKERNEL_RES_detelages_nums_get(i)==detelage_num) 
			break;
	}
	return i;
}

int TSKERNEL_MOD_detelages_getPosition(int detelage_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_detelages_isValidNum(detelage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_DETELAGE;
	//Execution
	return TSKERNEL_RES_detelages_positions_get(TSKERNEL_MOD_detelages_getOffset(detelage_num));
}

int TSKERNEL_MOD_detelages_isValidPosition(int pos){
	if (pos != DETELAGES_POSITION_OFF && pos != DETELAGES_POSITION_ON){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"detelages : invalid position %d",pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_DETELAGE_POSITION;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_detelages_isValidNum(int detelage_num){
	int ok=0;
	int i=0;
	for (i=0;i<DETELAGES_NB_RAILS;i++){
		if (TSKERNEL_RES_detelages_nums_get(i)==detelage_num){
			ok=1;
			break;
		}
	}
	if (!ok){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"detelages : invalid detelage %d",detelage_num);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);		
		return TSKERNEL_INVALID_DETELAGE;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_detelages_setPosition(int detelage_num,int pos){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_detelages_isValidPosition(pos)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_DETELAGE_POSITION;
	if (TSKERNEL_MOD_detelages_isValidNum(detelage_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_DETELAGE;
	//Execution
	int ret=TSKERNEL_CMD_interface_sendTwo(pos,detelage_num);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"detelages : changing position for detelage %d to %d",detelage_num,pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_detelages_positions_set(TSKERNEL_MOD_detelages_getOffset(detelage_num),pos);
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

