/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_detelages.h                                     
** Description : Module noyau detelage                               
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_detelages.h
 * \brief Module : kernel, Module noyau detelages
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_DETELAGES_H
#define KERNEL_MOD_DETELAGES_H
#include "const_detelages.h"
#include "const_kernel.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_detelages_nums;
	static SEM* TSKERNEL_SEM_detelages_positions;
#else
	static sem_t TSKERNEL_SEM_detelages_nums;
	static sem_t TSKERNEL_SEM_detelages_positions;
#endif

/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_detelages_positions_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_detelages_positions_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_detelages_positions_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_detelages_positions_get(int i);
/**
 * \fn int TSKERNEL_RES_detelages_nums_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_detelages_nums_get(int i);

/*
TSKERNEL_MOD_detelages
*/
static int TSKERNEL_DETELAGES_RAILS_POSITIONS[DETELAGES_NB_RAILS]={DETELAGES_POSITION_OFF,
DETELAGES_POSITION_OFF,DETELAGES_POSITION_OFF};

/**
 * \fn int TSKERNEL_MOD_detelages_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_detelages_load(void);
/**
 * \fn int TSKERNEL_MOD_detelages_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_detelages_unload(void);

/**
 * \fn int TSKERNEL_MOD_detelages_getOffset(int detelage_num)
 * \brief Renvoie l'offset du rail de detelage
 * \param detelage_num numero du rail
 * \return int offset
 */
int TSKERNEL_MOD_detelages_getOffset(int detelage_num);
/**
 * \fn int TSKERNEL_MOD_detelages_getPosition(int detelage_num)
 * \brief Renvoie la position du rail de detelage
 * \param detelage_num numero du rail
 * \return int position
 */
int TSKERNEL_MOD_detelages_getPosition(int detelage_num);
/**
 * \fn int TSKERNEL_MOD_detelages_isValidPosition(int pos)
 * \brief Teste la validite de la position
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_detelages_isValidPosition(int pos);
/**
 * \fn int TSKERNEL_MOD_detelages_isValidNum(int detelage_num)
 * \brief Teste la validite du numero
 * \param detelage_num numero du rail
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_detelages_isValidNum(int detelage_num);
/**
 * \fn int TSKERNEL_MOD_detelages_setPosition(int detelage_num,int pos)
 * \brief Change la position du rail
 * \param detelage_num numero du rail
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_detelages_setPosition(int detelage_num,int pos);

#endif
