/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_receptivities.h                                     
** Description : Module noyau receptivites                                  
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_receptivities.h
 * \brief Module : kernel, Module noyau receptivites
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_RECEPTIVITIES_H
#define KERNEL_MOD_RECEPTIVITIES_H
#include "const_receptivities.h"
#include "const_kernel.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_receptivities;
#else
	static sem_t TSKERNEL_SEM_receptivities;
#endif

/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_receptivities_set(int i,int j,int value)
 * \brief Change la ressource
 * \param i indice d'abscisse
 * \param j indice de colonne
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_receptivities_set(int i,int j,int value);
/**
 * \fn int TSKERNEL_RES_receptivities_get(int i,int j)
 * \brief Renvoie la valeur de la ressource
 * \param i indice d'abscisse
 * \param j indice de colonne
 * \return int valeur
 */
int TSKERNEL_RES_receptivities_get(int i,int j);

/*
TSKERNEL_MOD_receptivities
*/
static int TSKERNEL_RECEPTIVITIES[RECEPTIVITIES_READ_ALL_NB][RECEPTIVITIES_BITS_NB] ={
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT },
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT},
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT },
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT },
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT },
{ RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT, RECEPTIVITIES_STATE_INIT }
};

/**
 * \fn int TSKERNEL_MOD_receptivities_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_load(void);
/**
 * \fn int TSKERNEL_MOD_receptivities_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_unload(void);

/**
 * \fn int TSKERNEL_MOD_receptivities_get(int octet,int bit)
 * \brief Renvoie la receptivite
 * \param octet octet
 * \param bit bit
 * \return int receptivite
 */
int TSKERNEL_MOD_receptivities_get(int octet,int bit);
/**
 * \fn int TSKERNEL_MOD_receptivities_set(int octet,int bit,int value)
 * \brief Renvoie la receptivite
 * \param octet octet
 * \param bit bit
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_set(int octet,int bit,int value);
/**
 * \fn int TSKERNEL_MOD_receptivities_refreshAll(int clearOnExit)
 * \brief Rafraichit toutes les receptivites
 * \param clearOnExit parametre d'effacement
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_refreshAll(int clearOnExit);
/**
 * \fn int TSKERNEL_MOD_receptivities_isValidDecoder(int decoder_num)
 * \brief Teste le numero d'un decodeur
 * \param decoder_num numero de decoder
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_isValidDecoder(int decoder_num);
/**
 * \fn int TSKERNEL_MOD_receptivities_refresh(int decoder_num,int clearOnExit)
 * \brief Rafraichit les receptivites d'un decodeur
 * \param decoder_num numero de decoder
 * \param clearOnExit parametre d'effacement
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_receptivities_refresh(int decoder_num,int clearOnExit);

#endif
