/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                       
** Fichier : kernel_mod_serial.h                                     
** Description : Gestion de la ligne serie                              
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_serial.h
 * \brief Module : kernel, Module noyau gestion de la ligne serie
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_SERIAL_H
#define	KERNEL_MOD_SERIAL_H
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include "const_kernel.h"
#include "const_serial.h"
#ifdef TS_RTAI
	#include <sys/mman.h>
	#include <rtai_serial.h>
	#include <rtai_lxrt.h>
	#include <rtai_sem.h>
#else
	#include <termios.h>
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_serial_lock;
#else
	static sem_t TSKERNEL_SEM_serial_lock;
#endif


/*
TSKERNEL_MOD_serial
*/

static int TSKERNEL_MOD_serial_line=0;

/**
 * \fn int TSKERNEL_MOD_serial_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_serial_load(void);
/**
 * \fn int TSKERNEL_MOD_serial_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_serial_unload(void);

/**
 * \fn int TSKERNEL_MOD_serial_send(int lg,char* str)
 * \brief Envoie sur la ligne serie
 * \param lg longueur
 * \param str chaine de caracteres
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_serial_send(int lg,char* str);
/**
 * \fn int TSKERNEL_MOD_serial_read(void)
 * \brief Lecture sur la ligne serie
 * \return int caractere
 */
int TSKERNEL_MOD_serial_read(void);
/**
 * \fn int TSKERNEL_MOD_serial_cleanBuffer(void)
 * \brief Vide le buffer de communication
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_serial_cleanBuffer(void);

#endif
