/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_signalisation.c                                     
** Description : Module noyau signalisation                                  
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_signalisation.c
 * \brief Module : kernel, Module noyau signalisation
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_signalisation.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_signalisation_states_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_signalisation_states);
	#else
		sem_wait(&TSKERNEL_SEM_signalisation_states);
	#endif
	TSKERNEL_SIGNAUX_STATES[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_signalisation_states);
	#else
		sem_post(&TSKERNEL_SEM_signalisation_states);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_signalisation_states_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_signalisation_states);
	#else
		sem_wait(&TSKERNEL_SEM_signalisation_states);
	#endif
	value=TSKERNEL_SIGNAUX_STATES[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_signalisation_states);
	#else
		sem_post(&TSKERNEL_SEM_signalisation_states);
	#endif
	return value;
}

int TSKERNEL_RES_signalisation_nums_get(int i){
	int value=0;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_signalisation_nums);
	#else
		sem_wait(&TSKERNEL_SEM_signalisation_nums);
	#endif
	value=SIGNALISATION_NUMS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_signalisation_nums);
	#else
		sem_post(&TSKERNEL_SEM_signalisation_nums);
	#endif
	return value;
}

/*
TSKERNEL_MOD_signalisation
*/
int TSKERNEL_MOD_signalisation_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"signalisation : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_signalisation_nums=rt_sem_init(TSKERNEL_SEM_signalisation_nums,1);
		TSKERNEL_SEM_signalisation_states=rt_sem_init(TSKERNEL_SEM_signalisation_states,1);
	#else
		sem_init(&TSKERNEL_SEM_signalisation_nums,0,1);
		sem_init(&TSKERNEL_SEM_signalisation_states,0,1);
	#endif
	int i=0;
	//Tous en verts
	for(i=0;i<SIGNALISATION_NB_SIGNAUX;i++){
		TSKERNEL_MOD_signalisation_setState(TSKERNEL_RES_signalisation_nums_get(i),SIGNALISATION_COLOR_GREEN);
	}
	//Tous en rouge
	for(i=0;i<SIGNALISATION_NB_SIGNAUX;i++){
		TSKERNEL_MOD_signalisation_setState(TSKERNEL_RES_signalisation_nums_get(i),SIGNALISATION_COLOR_RED);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"signalisation : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_signalisation_unload(void){
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_signalisation_nums);
		rt_sem_delete(TSKERNEL_SEM_signalisation_states);
	#else
		sem_destroy(&TSKERNEL_SEM_signalisation_nums);
		sem_destroy(&TSKERNEL_SEM_signalisation_states);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"signalisation : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_signalisation_getOffset(int signal_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_signalisation_isValidNum(signal_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_SIGNAL;
	//Execution
	int i=0;
	for (i=0;i<SIGNALISATION_NB_SIGNAUX;i++){
		if (TSKERNEL_RES_signalisation_nums_get(i)==signal_num)
			break;
	}
	return i;
}

int TSKERNEL_MOD_signalisation_isValidState(int state){
	if (state != SIGNALISATION_COLOR_GREEN && state != SIGNALISATION_COLOR_RED){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"signalisation : invalid signal color %d",state);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_SIGNAL_COLOR;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_signalisation_isValidNum(int signal_num){
	int i=0;
	for (i=0;i<SIGNALISATION_NB_SIGNAUX;i++){
		if (TSKERNEL_RES_signalisation_nums_get(i)==signal_num)
			return TSKERNEL_NORMAL_EXIT;
	}
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"signalisation : invalid signal %d",signal_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
	free(message);
	return TSKERNEL_INVALID_SIGNAL;
}

int TSKERNEL_MOD_signalisation_setState(int signal_num, int state){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_signalisation_isValidState(state)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_SIGNAL_COLOR;
	if (TSKERNEL_MOD_signalisation_isValidNum(signal_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_SIGNAL;
	//Execution
	int ret=TSKERNEL_CMD_interface_sendTwo(state,signal_num);
	if(ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"signalisation : state changed for signal %d to %d",signal_num,state);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_signalisation_states_set(TSKERNEL_MOD_signalisation_getOffset(signal_num),state);	
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

int TSKERNEL_MOD_signalisation_getState(int signal_num){
	if (TSKERNEL_MOD_signalisation_isValidNum(signal_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_SIGNAL;
	//Execution
	return TSKERNEL_RES_signalisation_states_get(TSKERNEL_MOD_signalisation_getOffset(signal_num));
}

