/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_signalisation.h                                     
** Description : Module noyau signalisation                                  
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_signalisation.h
 * \brief Module : kernel, Module noyau signalisation
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_SIGNALISATION_H
#define KERNEL_MOD_SIGNALISATION_H
#include "const_signalisation.h"
#include "const_kernel.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_signalisation_nums;
	static SEM* TSKERNEL_SEM_signalisation_states;
#else
	static sem_t TSKERNEL_SEM_signalisation_nums;
	static sem_t TSKERNEL_SEM_signalisation_states;
#endif
/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_signalisation_states_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_signalisation_states_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_signalisation_states_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_signalisation_states_get(int i);
/**
 * \fn int TSKERNEL_RES_signalisation_nums_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_signalisation_nums_get(int i);

/*
TSKERNEL_MOD_signalisation
*/
static int TSKERNEL_SIGNAUX_STATES[SIGNALISATION_NB_SIGNAUX]= {
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,SIGNALISATION_COLOR_RED,
SIGNALISATION_COLOR_RED};

/**
 * \fn int TSKERNEL_MOD_signalisation_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_signalisation_load(void);
/**
 * \fn int TSKERNEL_MOD_signalisation_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_signalisation_unload(void);

/**
 * \fn int TSKERNEL_MOD_signalisation_isValidState(int state)
 * \brief Teste la validite d'un etat
 * \param state etat
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_signalisation_isValidState(int state);
/**
 * \fn int TSKERNEL_MOD_signalisation_isValidNum(int signal_num)
 * \brief Teste le numero de signal
 * \param signal_num numero de signal
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_signalisation_isValidNum(int signal_num);
/**
 * \fn int TSKERNEL_MOD_signalisation_setState(int signal_num, int state)
 * \brief Modifie l'etat du signal
 * \param signal_num numero de signal
 * \param state etat
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_signalisation_setState(int signal_num, int state);
/**
 * \fn int TSKERNEL_MOD_signalisation_getState(int signal_num)
 * \brief Renvoie l'etat du signal
 * \param signal_num numero de signal
 * \return int etat du signal
 */
int TSKERNEL_MOD_signalisation_getState(int signal_num);

#endif
