/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                       
** Fichier : kernel_mod_socket.h                                     
** Description : Gestion des sockets                           
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_socket.h
 * \brief Module : kernel, Module noyau socket
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_SOCKET_H
#define	KERNEL_MOD_SOCKET_H
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include "const_kernel.h"
#include "const_socket.h"
#include "const_api.h"

#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_socket_lock;
#else
	static sem_t TSKERNEL_SEM_socket_lock;
#endif

/*
TSKERNEL_MOD_socket
*/
int TSKERNEL_MOD_socket_recv_socket_id;

/**
 * \fn int TSKERNEL_MOD_socket_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_load(void);
/**
 * \fn int TSKERNEL_MOD_socket_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_unload(void);

/**
 * \fn int TSKERNEL_MOD_socket_createTcpClient(const char *hostname, int port)
 * \brief Creation d'un client TCP
 * \param hostname nom du serveur
 * \param port port
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_createTcpClient(const char *hostname, int port);
/**
 * \fn int TSKERNEL_MOD_socket_createTcpServer(int port, int nb_max_clients)
 * \brief Creation d'un serveur TCP
 * \param port port
 * \param nb_max_clients nombre de connexion maximal
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_createTcpServer(int port, int nb_max_clients);
/**
 * \fn int TSKERNEL_MOD_socket_read(char* recv_c)
 * \brief Lecture sur socket
 * \param recv_c caractere lu
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_read(char* recv_c);
/**
 * \fn int TSKERNEL_MOD_socket_send_1int(int op_code,int param1)
 * \brief Envoi sur socket d'un opcode et d'un integer
 * \param op_code code operation
 * \param param1 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_send_1int(int op_code,int param1);
/**
 * \fn int TSKERNEL_MOD_socket_sendAnswer(int param1)
 * \brief Envoi d'une reponse sur socket et d'un integer
 * \param param1 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_socket_sendAnswer(int param1);
#endif
