/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_trains.c                                     
** Description : Module noyau trains                             
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_trains.c
 * \brief Module : kernel, Module noyau trains
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_trains.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_trains_positions_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_positions);
	#else
		sem_wait(&TSKERNEL_SEM_trains_positions);
	#endif
	TSKERNEL_TRAINS_POSITIONS[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_positions);
	#else
		sem_post(&TSKERNEL_SEM_trains_positions);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_trains_positions_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_positions);
	#else
		sem_wait(&TSKERNEL_SEM_trains_positions);
	#endif
	value=TSKERNEL_TRAINS_POSITIONS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_positions);
	#else
		sem_post(&TSKERNEL_SEM_trains_positions);
	#endif
	return value;
}

int TSKERNEL_RES_trains_speeds_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_speeds);
	#else
		sem_wait(&TSKERNEL_SEM_trains_speeds);
	#endif
	TSKERNEL_TRAINS_SPEEDS[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_speeds);
	#else
		sem_post(&TSKERNEL_SEM_trains_speeds);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_trains_speeds_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_speeds);
	#else
		sem_wait(&TSKERNEL_SEM_trains_speeds);
	#endif
	value=TSKERNEL_TRAINS_SPEEDS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_speeds);
	#else
		sem_post(&TSKERNEL_SEM_trains_speeds);
	#endif
	return value;
}

int TSKERNEL_RES_trains_actives_set(int i,int value){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_actives);
	#else
		sem_wait(&TSKERNEL_SEM_trains_actives);
	#endif
	TSKERNEL_TASKS_trains_actives[i]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_actives);
	#else
		sem_post(&TSKERNEL_SEM_trains_actives);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_trains_actives_get(int i){
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_actives);
	#else
		sem_wait(&TSKERNEL_SEM_trains_actives);
	#endif
	value=TSKERNEL_TASKS_trains_actives[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_actives);
	#else
		sem_post(&TSKERNEL_SEM_trains_actives);
	#endif
	return value;
}

int TSKERNEL_RES_trains_nums_get(int i){
	int value=0;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_trains_nums);
	#else
		sem_wait(&TSKERNEL_SEM_trains_nums);
	#endif
	value=TRAINS_NUMS[i];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_trains_nums);
	#else
		sem_post(&TSKERNEL_SEM_trains_nums);
	#endif
	return value;
}

/*
TSKERNEL_MOD_trains
*/
int TSKERNEL_MOD_trains_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"trains : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_trains_nums=rt_sem_init(TSKERNEL_SEM_trains_nums,1);
		TSKERNEL_SEM_trains_speeds=rt_sem_init(TSKERNEL_SEM_trains_speeds,1);
		TSKERNEL_SEM_trains_positions=rt_sem_init(TSKERNEL_SEM_trains_positions,1);
		TSKERNEL_SEM_trains_actives=rt_sem_init(TSKERNEL_SEM_trains_actives,1);
	#else
		sem_init(&TSKERNEL_SEM_trains_nums,0,1);
		sem_init(&TSKERNEL_SEM_trains_speeds,0,1);
		sem_init(&TSKERNEL_SEM_trains_positions,0,1);
		sem_init(&TSKERNEL_SEM_trains_actives,0,1);
	#endif
	int i=0;
	for(i=0;i<TRAINS_NB_LOCOMOTIVES;i++){
		TSKERNEL_MOD_trains_setSpeed(TSKERNEL_RES_trains_nums_get(i),TRAINS_CMD_NOFCT_ARRET,0);
		TSKERNEL_MOD_trains_setPosition(TSKERNEL_RES_trains_nums_get(i),BLOCKS_NOT_ASSIGNED);
		TSKERNEL_RES_trains_actives_set(i,TRAINS_STATE_NOT_ACTIVE);
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"trains : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_unload(void){
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_trains_nums);
		rt_sem_delete(TSKERNEL_SEM_trains_speeds);
		rt_sem_delete(TSKERNEL_SEM_trains_positions);
		rt_sem_delete(TSKERNEL_SEM_trains_actives);
	#else
		sem_destroy(&TSKERNEL_SEM_trains_nums);
		sem_destroy(&TSKERNEL_SEM_trains_speeds);
		sem_destroy(&TSKERNEL_SEM_trains_positions);
		sem_destroy(&TSKERNEL_SEM_trains_actives);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"trains : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_getOffset(int train_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	//Execution
	int i=0;
	for (i=0;i<TRAINS_NB_LOCOMOTIVES;i++){
		if (TSKERNEL_RES_trains_nums_get(i)==train_num) 
			break;
	}
	return i;
}

int TSKERNEL_MOD_trains_getSpeed(int train_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	//Execution
	return TSKERNEL_RES_trains_speeds_get(TSKERNEL_MOD_trains_getOffset(train_num));
}

int TSKERNEL_MOD_trains_getPosition(int train_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	//Execution
	return TSKERNEL_RES_trains_positions_get(TSKERNEL_MOD_trains_getOffset(train_num));
}

int TSKERNEL_MOD_trains_isValidTrain(int train_num){
	int i=0;
	for(i=0;i<TRAINS_NB_LOCOMOTIVES;i++){
		if (TSKERNEL_RES_trains_nums_get(i)==train_num) return TSKERNEL_NORMAL_EXIT;
	}
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"trains : invalid train %d",train_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
	free(message);
	return TSKERNEL_INVALID_TRAIN;
}

int TSKERNEL_MOD_trains_isValidSpeed(int speed,int fct){
	if (fct==0){
		if(speed>=TRAINS_CMD_NOFCT_ARRET && speed<=TRAINS_CMD_NOFCT_VITESSE_MAX)
			return TSKERNEL_NORMAL_EXIT;
		else {
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"trains : invalid speed %d",speed);
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
			free(message);
			return TSKERNEL_INVALID_TRAIN_SPEED;
		}
	} else if (fct==1){
		if(speed>=TRAINS_CMD_WITHFCT_ARRET && speed<=TRAINS_CMD_WITHFCT_VITESSE_MAX)
			return TSKERNEL_NORMAL_EXIT;
		else {
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"trains : invalid speed %d",speed);
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
			free(message);
			return TSKERNEL_INVALID_TRAIN_SPEED;
		}
	} else return TSKERNEL_INVALID_TRAIN_SPEED;
}

int TSKERNEL_MOD_trains_isValidFct(int fct){
	if (fct!=TRAINS_FCT_ON && fct!=TRAINS_FCT_OFF){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"trains : invalid function %d",fct);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_TRAIN_FCT;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_isValidPosition(int pos){
	if (pos<BLOCKS_NOT_ASSIGNED || pos>BLOCKS_G2){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"trains : invalid position %d",pos);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_TRAIN_POSITION;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_isActive(int train_num){
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	int ret=TSKERNEL_RES_trains_actives_get(TSKERNEL_MOD_trains_getOffset(train_num));
	if (ret==TRAINS_STATE_ACTIVE)
		return TSKERNEL_NORMAL_EXIT;
	else return TSKERNEL_INACTIVE_TRAIN;
}

int TSKERNEL_MOD_trains_setSpeed(int train_num,int speed,int fct){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	if (TSKERNEL_MOD_trains_isValidFct(fct)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN_FCT;
	if (TSKERNEL_MOD_trains_isValidSpeed(speed,fct)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN_SPEED;
	//Execution
	int ret=TSKERNEL_CMD_interface_sendTwo(speed,train_num);
	if (ret==TSKERNEL_NORMAL_EXIT){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"trains : changing speed for train %d to %d with fct %d",train_num,speed,fct);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
		free(message);
		TSKERNEL_RES_trains_speeds_set(TSKERNEL_MOD_trains_getOffset(train_num),speed);
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return ret;
}

int TSKERNEL_MOD_trains_setPosition(int train_num,int pos){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	if (TSKERNEL_MOD_trains_isValidPosition(pos)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN_POSITION;
	//Execution
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"trains : changing position for train %d to %d",train_num,pos);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);
	TSKERNEL_RES_trains_positions_set(TSKERNEL_MOD_trains_getOffset(train_num),pos);
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_reverse(int train_num,int fct){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	if (TSKERNEL_MOD_trains_isValidFct(fct)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN_FCT;
	//Execution
	if (fct==TRAINS_FCT_OFF){
		int ret=TSKERNEL_CMD_interface_sendTwo(TRAINS_CMD_NOFCT_INVERSION,train_num);
		if (ret==TSKERNEL_NORMAL_EXIT){
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"trains : changing side for train %d with fct %d",train_num,fct);
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
			free(message);
			TSKERNEL_RES_trains_speeds_set(TSKERNEL_MOD_trains_getOffset(train_num),TRAINS_CMD_NOFCT_INVERSION);
		}
		if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
		return ret;
	} else if (fct==TRAINS_FCT_ON){
		int ret=TSKERNEL_CMD_interface_sendTwo(TRAINS_CMD_WITHFCT_INVERSION,train_num);
		if (ret==TSKERNEL_NORMAL_EXIT){
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"trains : changing side for train %d with fct %d",train_num,fct);
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
			free(message);
			TSKERNEL_RES_trains_speeds_set(TSKERNEL_MOD_trains_getOffset(train_num),TRAINS_CMD_WITHFCT_INVERSION);
		}
		if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
		return ret;
	} else return TSKERNEL_INVALID_TRAIN_FCT;
}

int TSKERNEL_MOD_trains_createItinerary(int train_num,int startBlock_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	if(TSKERNEL_MOD_blocks_isValidBlock(startBlock_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_BLOCK;
	if (TSKERNEL_MOD_trains_isActive(train_num)==TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_ACTIVE_TRAIN;
	//Execution
	int train_offset=TSKERNEL_MOD_trains_getOffset(train_num);

	TSKERNEL_MOD_trains_setSpeed(train_num,TRAINS_CMD_NOFCT_ARRET,TRAINS_FCT_OFF);
	TSKERNEL_MOD_trains_setPosition(train_num,startBlock_num);
	TSKERNEL_RES_trains_actives_set(train_offset,TRAINS_STATE_ACTIVE);

	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"trains : train %d task created",train_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);

	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_trains_deleteItinerary(int train_num){
	//Test des valeurs des parametres
	if (TSKERNEL_MOD_trains_isValidTrain(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_TRAIN;
	if (TSKERNEL_MOD_trains_isActive(train_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INACTIVE_TRAIN;
	//Execution
	int train_offset=TSKERNEL_MOD_trains_getOffset(train_num);
	TSKERNEL_RES_trains_actives_set(train_offset,TRAINS_STATE_NOT_ACTIVE);
	TSKERNEL_MOD_trains_setPosition(train_num,BLOCKS_NOT_ASSIGNED);
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"trains : train %d task delete",train_num);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_NORMAL_EXIT;
}
