/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_trains.h                                    
** Description : Module noyau trains                            
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_trains.h
 * \brief Module : kernel, Module noyau trains
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef KERNEL_MOD_TRAINS_H
#define KERNEL_MOD_TRAINS_H
#include "const_trains.h"
#include "const_blocks.h"
#include "const_kernel.h"
#include "const_tasks.h"
#include "tasks.h"
#include "kernel.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
	#include <pthread.h>
#endif
/*
TSKERNEL_TASKS
*/
static int TSKERNEL_TASKS_trains_actives[TRAINS_NB_LOCOMOTIVES]={TRAINS_STATE_NOT_ACTIVE,
TRAINS_STATE_NOT_ACTIVE,TRAINS_STATE_NOT_ACTIVE,TRAINS_STATE_NOT_ACTIVE,
TRAINS_STATE_NOT_ACTIVE,TRAINS_STATE_NOT_ACTIVE,TRAINS_STATE_NOT_ACTIVE,
TRAINS_STATE_NOT_ACTIVE};

/*
TSKERNEL_SEM
*/
#ifdef TS_RTAI
	static SEM* TSKERNEL_SEM_trains_nums;
	static SEM* TSKERNEL_SEM_trains_speeds;
	static SEM* TSKERNEL_SEM_trains_positions;
	static SEM* TSKERNEL_SEM_trains_actives;
#else
	static sem_t TSKERNEL_SEM_trains_nums;
	static sem_t TSKERNEL_SEM_trains_speeds;
	static sem_t TSKERNEL_SEM_trains_positions;
	static sem_t TSKERNEL_SEM_trains_actives;
#endif

/*
TSKERNEL_RES
*/
/**
 * \fn int TSKERNEL_RES_trains_speeds_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_trains_speeds_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_trains_speeds_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_trains_speeds_get(int i);
/**
 * \fn int TSKERNEL_RES_trains_positions_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_trains_positions_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_trains_positions_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_trains_positions_get(int i);
/**
 * \fn int TSKERNEL_RES_trains_actives_set(int i,int value)
 * \brief Change la ressource
 * \param i indice
 * \param value valeur
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_RES_trains_actives_set(int i,int value);
/**
 * \fn int TSKERNEL_RES_trains_actives_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_trains_actives_get(int i);
/**
 * \fn int TSKERNEL_RES_trains_nums_get(int i)
 * \brief Renvoie la valeur de la ressource
 * \param i indice
 * \return int valeur
 */
int TSKERNEL_RES_trains_nums_get(int i);

/*
TSKERNEL_MOD_trains
*/
static int TSKERNEL_TRAINS_SPEEDS[TRAINS_NB_LOCOMOTIVES]={TRAINS_CMD_NOFCT_ARRET,
TRAINS_CMD_NOFCT_ARRET,TRAINS_CMD_NOFCT_ARRET,TRAINS_CMD_NOFCT_ARRET,
TRAINS_CMD_NOFCT_ARRET,TRAINS_CMD_NOFCT_ARRET,TRAINS_CMD_NOFCT_ARRET,
TRAINS_CMD_NOFCT_ARRET};

static int TSKERNEL_TRAINS_POSITIONS[TRAINS_NB_LOCOMOTIVES]={BLOCKS_NOT_ASSIGNED,
BLOCKS_NOT_ASSIGNED,BLOCKS_NOT_ASSIGNED,BLOCKS_NOT_ASSIGNED,
BLOCKS_NOT_ASSIGNED,BLOCKS_NOT_ASSIGNED,BLOCKS_NOT_ASSIGNED,
BLOCKS_NOT_ASSIGNED};

/**
 * \fn int TSKERNEL_MOD_trains_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_load(void);
/**
 * \fn int TSKERNEL_MOD_trains_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_unload(void);

/**
 * \fn int TSKERNEL_MOD_trains_getOffset(int train_num)
 * \brief Renvoie l'offset du train
 * \param train_num numero du train
 * \return int offset
 */
int TSKERNEL_MOD_trains_getOffset(int train_num);
/**
 * \fn int TSKERNEL_MOD_trains_getSpeed(int train_num)
 * \brief Renvoie la vitesse du train
 * \param train_num numero du train
 * \return int vitesse
 */
int TSKERNEL_MOD_trains_getSpeed(int train_num);
/**
 * \fn int TSKERNEL_MOD_trains_getPosition(int train_num)
 * \brief Renvoie la position du train
 * \param train_num numero du train
 * \return int position
 */
int TSKERNEL_MOD_trains_getPosition(int train_num);
/**
 * \fn int TSKERNEL_MOD_trains_isValidTrain(int train_num)
 * \brief Teste le numero du train
 * \param train_num numero du train
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_isValidTrain(int train_num);
/**
 * \fn int TSKERNEL_MOD_blocks_isValidBlock(int block_num)
 * \brief Teste le numero du block
 * \param block_num numero de block
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_isValidSpeed(int speed,int fct);
/**
 * \fn int TSKERNEL_MOD_trains_isValidFct(int fct)
 * \brief Teste la validite de la fonction
 * \param fct fonction
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_isValidFct(int fct);
/**
 * \fn int TSKERNEL_MOD_trains_isValidPosition(int pos)
 * \brief Teste la position
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_isValidPosition(int pos);
/**
 * \fn int TSKERNEL_MOD_trains_isActive(int train_num)
 * \brief Teste si le train est actif
 * \param train_num numero du train
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_isActive(int train_num);
/**
 * \fn int TSKERNEL_MOD_trains_setSpeed(int train_num,int speed,int fct)
 * \brief Changement de la vitesse du train
 * \param train_num numero du train
 * \param speed vitesse
 * \param fct fonction
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_setSpeed(int train_num,int speed,int fct);
/**
 * \fn int TSKERNEL_MOD_trains_setPosition(int train_num,int pos)
 * \brief Change la position d'un train
 * \param train_num numero du train
 * \param pos position
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_setPosition(int train_num,int pos);
/**
 * \fn int TSKERNEL_MOD_trains_reverse(int train_num,int fct)
 * \brief Inverse le sens du train
 * \param train_num numero du train
 * \param fct fonction
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_reverse(int train_num,int fct);
/**
 * \fn int TSKERNEL_MOD_trains_createItinerary(int train_num,int startBlock_num)
 * \brief Creation d'itineraire de train
 * \param train_num numero du train
 * \param startBlock_num bloc de depart
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_createItinerary(int train_num,int startBlock_num);
/**
 * \fn int TSKERNEL_MOD_trains_deleteItinerary(int train_num)
 * \brief Destruction d'itineraire de train
 * \param train_num numero du train
 * \return int code etat (see const_kernel.h)
 */
int TSKERNEL_MOD_trains_deleteItinerary(int train_num);
#endif
