/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : tasks                                                      
** Fichier : tasks.c                                    
** Description : Module tasks                          
------------------------------------------------------------------------------*/
/**
 * \file tasks.c
 * \brief Module : tasks
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "tasks.h"
/*
TS_tasks
*/
void* TS_tasks_genericTask(int indx_arg,int rt_state,char* taskName,void (*taskBody)(void),void (*taskEnd)(void)){
	#ifdef TS_RTAI
		RTIME start_time, period;
		RTIME t0, t;
		RT_TASK *mytask;
		unsigned long mytask_name;
		int mytask_indx;

		mytask_indx = indx_arg;
		mytask_name = TS_TASKS_taskname(mytask_indx);
		TS_tasks_cpus_allowed = 1 - TS_tasks_cpus_allowed; 
	 	if (!(mytask = rt_task_init_schmod(mytask_name, 1, 0, 0, SCHED_FIFO, 1))){// << TS_tasks_cpus_allowed))) {
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"tasks : cannot init %s task",taskName);
			free(message);
			TS_trace(TS_TRACE_ERROR,message); 
			exit(TS_ERROR_EXIT);
		}
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"tasks : start %s task with address = %p",taskName,mytask);
		TS_trace(TS_TRACE_DEBUG,message);
		free(message);
		mlockall(MCL_CURRENT | MCL_FUTURE);

	 	if (rt_state==TS_TASKS_HARD_RT) {
			rt_make_hard_real_time();
		} else {
			rt_make_soft_real_time();
		}

		period = nano2count(TS_TASKS_PERIOD);
		start_time = rt_get_time() + nano2count(10000000);
		rt_task_make_periodic(mytask, start_time + (mytask_indx + 1)*period, TS_TASKS_NB*period);

	// start of task body
		{
	#endif
		if(taskBody!=NULL) taskBody();
	#ifdef TS_RTAI
		}
	// end of task body
	 	if (rt_state==TS_TASKS_HARD_RT) {
			rt_make_soft_real_time();
		}

		rt_task_delete(mytask);
	#endif
	if(taskEnd!=NULL) taskEnd();
	char* messageEnd=(char*)malloc(1024*sizeof(char));
	sprintf(messageEnd,"tasks : end %s task",taskName);
	free(messageEnd);
	TS_trace(TS_TRACE_DEBUG,messageEnd);
	return 0;
}

/*_generic*/
void* TS_tasks_generic(void* arg){
	TS_tasks_params_s params=*((TS_tasks_params_s*)arg);
	return	TS_tasks_genericTask(params.mytask_indx,params.rt_state,params.taskName,params.taskBody,params.taskEnd);
}

/*_kernel*/
void TS_tasks_kernel_body(void){
	//Kernel init + load modules
	TSKERNEL_pre_init();
	//Kernel infinite loop
	TSKERNEL_loop();
}

void TS_tasks_kernel_end(void){
	//Kernel + unload modules	
	//TSKERNEL_end();
}

void TS_tasks_gui_body(void){
	#ifndef TS_NOGUI	
		gtk_widget_show(TSGUI_mainWindow);

		//g_timeout_add(1000,TSGUI_updateCircuit,NULL);
		//g_timeout_add(3000,TSGUI_updateLog,NULL);
		
		//GUI main events loop
		gdk_threads_enter();
		gtk_main ();
		gdk_threads_leave();
	#else
		TSKERNEL_runSimul();
	#endif
}

void TS_tasks_exec(char *args[]){
	int pid;
	//Initialisation des répertoires d'entrées-sorties
	char* apps_target;

	apps_target=(char*)malloc(1024*sizeof(char));
	if (apps_target==NULL) exit(1);

	strcpy(apps_target,"./");

	strcat(apps_target,args[0]);

	//Execution
	switch (pid = fork ()){
		case -1:
			TS_trace(TS_TRACE_ERROR,"tasks : unable to launch fork()");
		case 0:
			execv(apps_target,args);
			TS_trace(TS_TRACE_ERROR,"tasks : error process forked");
	}
}
