/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : tasks                                                      
** Fichier : tasks.h                                    
** Description : Module tasks                          
------------------------------------------------------------------------------*/
/**
 * \file tasks.h
 * \brief Module : tasks
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef TASKS_H
#define TASKS_H
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <assert.h>
#include "kernel.h"
#include "const_global.h"
#include "const_kernel.h"
#include "const_tasks.h"

/*
TS_tasks
*/
typedef struct TS_tasks_params_s {
	int mytask_indx;
	int rt_state;
	char* taskName;
	void (*taskBody)(void);
	void (*taskEnd)(void);
} TS_tasks_params_s;

static int TS_tasks_argc;
static char** TS_tasks_argv;

static pthread_t TS_tasks[TS_TASKS_NB];
static int TS_tasks_indx[TS_TASKS_NB];
static int TS_tasks_cpus_allowed;

/*_generic*/
/**
 * \fn void* TS_tasks_genericTask(int indx_arg,int rt_state,char* taskName,void (*taskBody)(void),void (*taskEnd)(void))
 * \brief Creation d'une tache generique
 * \param index_arg numero de tache
 * \param rt_state etat temps reel
 * \param taskName nom de la tache
 * \param taskBody pointeur sur fonction de corps
 * \param taskEnd pointeur sur fonction de fin
 * \return void* pointeur sur tache
 */
void* TS_tasks_genericTask(int indx_arg,int rt_state,char* taskName,void (*taskBody)(void),void (*taskEnd)(void));
/**
 * \fn void* TS_tasks_generic(void* arg)
 * \brief Creation d'une tache generique
 * \param arg structure parametree de tache
 * \return void* pointeur sur tache
 */
void* TS_tasks_generic(void* arg);

/*_kernel*/
/**
 * \fn void TS_tasks_kernel_body(void)
 * \brief Fonction corps de la tache noyau
 * \return void
 */
void TS_tasks_kernel_body(void);
/**
 * \fn void TS_tasks_kernel_body(void)
 * \brief Fonction de fin de la tache noyau
 * \return void
 */
void TS_tasks_kernel_end(void);

/*_gui*/
/**
 * \fn void TS_tasks_gui_body(void)
 * \brief Corps de la tache gui
 * \return void
 */
void TS_tasks_gui_body(void);

/**
 * \fn void TS_tasks_exec(char *args[])
 * \brief Excution d'une tache hors processus
 * \param args argument du programme
 * \return void
 */
void TS_tasks_exec(char *args[]);
#endif
