/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : api                                                        
** Fichier : api_usr.c                                     
** Description : API to use kernel for endusers, all API calls will be 
**               translated to kernel calls
------------------------------------------------------------------------------*/
/**
 * \file api_usr.c
 * \brief Module : api, API to use kernel for endusers, all API calls will be 
 *               translated to kernel calls
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "api_usr.h"
/*
TSAPI
*/
int TSAPI_init(void){
	return TSUSR_NORMAL_EXIT;
}
int TSAPI_end(void){
	int ret=0;
	return ret;
}

int TSAPI_trace(int type,char* message){
	if (type<=TSUSR_trace_level){
		switch(type){
			case TSUSR_TRACE_ERROR:
				printf("[TSAPI_ERROR  ] %s\n",message);
			break;
			case TSUSR_TRACE_INFO:
				printf("[TSAPI_INFO   ] %s\n",message);
			break;
			case TSUSR_TRACE_DEBUG:
				printf("[TSAPI_DEBUG  ] %s\n",message);
			break;
		}
	}
	return TSUSR_NORMAL_EXIT;
}

/*
TSAPI_aiguillages
*/
int TSAPI_aiguillages_setPosition(char* aiguillage_num,char* aiguillage_pos){
	int ret=0;
	TSUSR_socket_send_2str(API_OPCODE_AIGUILLAGES_SETPOS,aiguillage_num,aiguillage_pos);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TSAPI_blocks
*/
int TSAPI_blocks_mutex_P(int block_num,int train_num){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_BLOCKS_MUTEX_P,block_num,train_num);
	ret=TSUSR_socket_readAnswer();
	TSUSR_blocks_mutex_P(block_num,train_num);
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_blocks_mutex_V(int block_num,int train_num){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_BLOCKS_MUTEX_V,block_num,train_num);
	ret=TSUSR_socket_readAnswer();
	TSUSR_blocks_mutex_V(block_num,train_num);
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TSAPI_circuit
*/
int TSAPI_circuit_relachement(void){
	int ret=0;
	TSUSR_socket_send(API_OPCODE_CIRCUIT_RELACHEMENT);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_circuit_disjonction(void){
	int ret=0;
	TSUSR_socket_send(API_OPCODE_CIRCUIT_DISJONCTION);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_circuit_conjonction(void){
	int ret=0;
	TSUSR_socket_send(API_OPCODE_CIRCUIT_CONJONCTION);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TASPI_detelages
*/
int TSAPI_detelages_getPosition(int detelage_num){
	int ret=0;
	TSUSR_socket_send_1int(API_OPCODE_DETELAGES_GETPOS,detelage_num);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_detelages_setPosition(char* detelage_name,int pos){
	int ret=0;
	TSUSR_socket_send_2strint(API_OPCODE_DETELAGES_SETPOS,detelage_name,pos);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TASPI_receptivities
*/
int TSAPI_receptivities_get(int octet,int bit){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_RECEPTIVITIES_GET,octet,bit);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TSAPI_signalisation
*/
int TSAPI_signalisation_setState(int signal_num, int state){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_SIGNALISATION_SET,signal_num,state);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_signalisation_getState(int signal_num){
	int ret=0;
	TSUSR_socket_send_1int(API_OPCODE_SIGNALISATION_GET,signal_num);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

/*
TSAPI_trains
*/
int TSAPI_trains_getSpeed(int train_num){
	int ret=0;
	TSUSR_socket_send_1int(API_OPCODE_TRAINS_GETSPEED,train_num);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_trains_getPosition(int train_num){
	int ret=0;
	TSUSR_socket_send_1int(API_OPCODE_TRAINS_GETPOS,train_num);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_trains_setSpeed(int train_num,int speed,int fct){
	int ret=0;
	TSUSR_socket_send_3int(API_OPCODE_TRAINS_SETSPEED,train_num,speed,fct);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}
int TSAPI_trains_reverse(int train_num,int fct){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_TRAINS_REVERSE,train_num,fct);
	ret=TSUSR_socket_readAnswer();
	#ifdef TSUSR_RTAI
		rt_task_yield(); 
	#else
		pthread_yield(); 
	#endif
	return ret;
}

int TSAPI_trains_createItinerary(int train_num,int startBlock_num,int cyclic_state,void (*initItinerary)(void),void (*itinerary)(void),void (*endItinerary)(void)){
	int ret=0;
	TSUSR_socket_send_2int(API_OPCODE_TRAINS_CREATE,train_num,startBlock_num);
	ret=TSUSR_socket_readAnswer();
	int i=0;
	for(i=0;i<TRAINS_NB_LOCOMOTIVES;i++){
		if (TRAINS_NUMS[i]==train_num) break;
	}
	TSUSR_tasks_params_s train_params;
	train_params.mytask_indx=TSUSR_tasks_indx[i];
	train_params.rt_state=TSUSR_TASKS_SOFT_RT;
	train_params.cyclic_state=TSUSR_TASKS_CYCLIC;
	char* train_name=(char*) malloc(255*sizeof(char));
	sprintf(train_name,"train%d",TRAINS_NUMS[i]);
	train_params.taskName=train_name;
	train_params.taskInit=initItinerary;
	train_params.taskBody=itinerary;
	train_params.taskEnd=endItinerary;
	#ifdef TSUSR_RTAI
		if (!(TSUSR_tasks[i] = rt_thread_create(TSUSR_tasks_generic, &train_params,10000))) {
	#else
		if (pthread_create(&TSUSR_tasks[i],0,TSUSR_tasks_generic, &train_params)!=0) {
	#endif
		TSUSR_trace(TSUSR_TRACE_ERROR,"usr : cannot create some train thread"); 
		exit(TSUSR_ERROR_EXIT);
	}
	return ret;
}
int TSAPI_trains_deleteItinerary(int train_num){
	int ret=0;
	TSUSR_socket_send_1int(API_OPCODE_TRAINS_DELETE,train_num);
	ret=TSUSR_socket_readAnswer();
	return ret;
}

