/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr                                                       
** Fichier : usr_blocks.c                                     
** Description : Module utilisateur blocs
------------------------------------------------------------------------------*/
/**
 * \file usr_blocks.c
 * \brief Module : usr, gestion des blocs utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "usr_blocks.h"

/*
TSUSR_blocks
*/
int TSUSR_blocks_load(void){
	TSUSR_trace(TSUSR_TRACE_INFO,"blocks : load begin");
	//Initialisation des semaphores
	int i=0;
	for(i=0;i<BLOCKS_NB_BLOCKS;i++){
		#ifdef TS_RTAI
			TSUSR_SEM_blocks_sem[i]=rt_sem_init(TSUSR_SEM_blocks_sem[i],1);
		#else
			sem_init(&TSUSR_SEM_blocks_sem[i],0,1);
		#endif
	}
	TSUSR_trace(TSUSR_TRACE_INFO,"blocks : load end");
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_blocks_unload(void){
	//Destruction des semaphores
	int i=0;
	for(i=0;i<BLOCKS_NB_BLOCKS;i++){
		#ifdef TS_RTAI
			rt_sem_delete(TSUSR_SEM_blocks_sem[i]);
		#else
			sem_destroy(&TSUSR_SEM_blocks_sem[i]);
		#endif
	}
	TSUSR_trace(TSUSR_TRACE_INFO,"blocks : unload");
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_blocks_isValidBlock(int block_num){
	if (block_num<BLOCKS_NOT_ASSIGNED || block_num>BLOCKS_G2){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"blocks : invalid block %d",block_num);
		TSUSR_trace(TSUSR_TRACE_ERROR,message);
		free(message);
		return TSUSR_INVALID_BLOCK;
	}
	else return TSUSR_NORMAL_EXIT;
}


int TSUSR_blocks_mutex_P(int block_num,int train_num){
	if(TSUSR_blocks_isValidBlock(block_num)!=TSUSR_NORMAL_EXIT)
		return TSUSR_INVALID_BLOCK;
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_blocks_sem[block_num]);
	#else
		sem_wait(&TSUSR_SEM_blocks_sem[block_num]);
	#endif
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"blocks : P(block %d) for train %d",block_num,train_num);
	TSUSR_trace(TSUSR_TRACE_INFO,message);
	free(message);
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_blocks_mutex_V(int block_num,int train_num){
	if(TSUSR_blocks_isValidBlock(block_num)!=TSUSR_NORMAL_EXIT)
		return TSUSR_INVALID_BLOCK;
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_blocks_sem[block_num]);
	#else
		sem_post(&TSUSR_SEM_blocks_sem[block_num]);
	#endif
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"blocks : V(block %d) for train %d",block_num,train_num);
	TSUSR_trace(TSUSR_TRACE_INFO,message);
	free(message);
	return TSUSR_NORMAL_EXIT;
}

