/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr                                                       
** Fichier : usr_blocks.h                                    
** Description : Module utilisateur blocs
------------------------------------------------------------------------------*/
/**
 * \file usr_blocks.h
 * \brief Module : usr, gestion des blocs utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef USR_BLOCKS_H
#define USR_BLOCKS_H
#include "const_blocks.h"
#include "usr_const_global.h"
#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSUSR_SEM
*/
#ifdef TS_RTAI
	static SEM* TSUSR_SEM_blocks_sem[BLOCKS_NB_BLOCKS];//Attention indice 0 non utilisé car block 0 = hors voie et plusieurs trains peuvent être hors voie
#else
	static sem_t TSUSR_SEM_blocks_sem[BLOCKS_NB_BLOCKS];//Attention indice 0 non utilisé car block 0 = hors voie et plusieurs trains peuvent être hors voie
#endif

/*
TSUSR_blocks
*/
/**
 * \fn int TSUSR_blocks_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_blocks_load(void);
/**
 * \fn int TSUSR_blocks_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_blocks_unload(void);

/**
 * \fn int TSUSR_blocks_isValidBlock(int block_num)
 * \brief Teste le numero du block
 * \param block_num numero de block
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_blocks_isValidBlock(int block_num);
/**
 * \fn int TSUSR_blocks_mutex_P(int block_num,int train_num)
 * \brief Prise de mutex sur un block
 * \param block_num numero de block
 * \param train_num numero de train
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_blocks_mutex_P(int block_num,int train_num);
/**
 * \fn int TSUSR_blocks_mutex_V(int block_num,int train_num)
 * \brief Liberation de mutex sur un block
 * \param block_num numero de block
 * \param train_num numero de train
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_blocks_mutex_V(int block_num,int train_num);

#endif
