/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr                                                      
** Fichier : usr_socket.c                                     
** Description : Gestion de la socket                             
------------------------------------------------------------------------------*/
/**
 * \file usr_socket.c
 * \brief Module : usr, gestion des sockets utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "usr_socket.h"

/*
TSUSR_socket
*/
int TSUSR_socket_load(void){
	TSUSR_trace(TSUSR_TRACE_INFO,"socket : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSUSR_SEM_socket_lock=rt_sem_init(TSUSR_SEM_socket_lock,1);
	#else
		sem_init(&TSUSR_SEM_socket_lock,0,1);
	#endif

	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	//Init SEND socket
	TSUSR_socket_send_socket_id = TSUSR_socket_createTcpServer(SOCKET_PORT_SEND,1);
	if (-1 == (TSUSR_socket_send_client_id = accept (TSUSR_socket_send_socket_id, NULL, 0))){
      		return TSUSR_INVALID_SOCKET;
    	}
  	shutdown (TSUSR_socket_send_socket_id, 2);
	close (TSUSR_socket_send_socket_id);

	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	TSUSR_trace(TSUSR_TRACE_INFO,"socket : load end");
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_unload(void){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif

	//Close socket
	shutdown (TSUSR_socket_send_client_id, 2);
	close (TSUSR_socket_send_client_id);

	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif

	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSUSR_SEM_socket_lock);
	#else
		sem_destroy(&TSUSR_SEM_socket_lock);
	#endif
	TSUSR_trace(TSUSR_TRACE_INFO,"socket : unload");
	return	TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_createTcpServer(int port, int nb_max_clients){
	int socket_id;
	int optval = 1;
	struct sockaddr_in sockname;

	// Création de la socket
	if (-1 == (socket_id = socket (PF_INET, SOCK_STREAM, 0))){
		TSUSR_trace(TSUSR_TRACE_ERROR,"socket : unable to create socket");
		return TSUSR_INVALID_SOCKET;
	}
	/* Changement d'un paramètre de la socket pour permettre une réutilisation
	 * immédiate après sa fermeture
	 */
	setsockopt (socket_id, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof (int));

	// Affectation d'une adresse
	memset ((char *) &sockname, 0, sizeof (struct sockaddr_in));
	sockname.sin_family = AF_INET;
	sockname.sin_port = htons (port);
	sockname.sin_addr.s_addr = htonl (INADDR_ANY);
	if (-1 ==(bind(socket_id, (struct sockaddr *) &sockname,sizeof (struct sockaddr_in)))){
		TSUSR_trace(TSUSR_TRACE_ERROR,"socket : unable to bind socket");
		return TSUSR_INVALID_SOCKET;
	}

	// Mise en écoute de la socket
	if (-1 == (listen (socket_id, nb_max_clients))){
		TSUSR_trace(TSUSR_TRACE_ERROR,"socket : unable to listen socket");
		return TSUSR_INVALID_SOCKET;
	}
	return (socket_id);
}

int TSUSR_socket_read(char* recv_c){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	int ret=0;
	ret=read(TSUSR_socket_send_client_id,recv_c, sizeof(char));
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return ret;
}

int TSUSR_socket_readAnswer(void){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char recv_c;
	int recv_state=-1;
	int op_code=-1;
	int param1=-1;
	char* answer=(char*)malloc(1024*sizeof(char));
	strcpy(answer,"");

	while (1 == read(TSUSR_socket_send_client_id,&recv_c, sizeof(char))){
		//Parse Query
		char* recv_str=(char*)malloc(5*sizeof(char));
		sprintf(recv_str,"%c",recv_c);
		strcat(answer,recv_str);

		if (recv_c==API_QUERY_START_CHAR){//Begin op
			recv_state=1;
		} else if (recv_c==API_QUERY_END_CHAR){//End op
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"socket : simul recv command : %s",answer);
			TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
			free(message);

			char** querySplitted=str_split(answer,API_DELIMITER_STR);
			op_code=atoi(querySplitted[1]);
			param1=atoi(querySplitted[2]);
			strcpy(answer,"");
			recv_state=0;
			break;
		}	
	}
	free(answer);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	if (op_code==API_OPCODE_ANSWER) return param1;
	else return TSUSR_INVALID_SOCKET_ANSWER;
}

int TSUSR_socket_send_3int(int op_code,int param1,int param2,int param3){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%d%c%d%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,param2,API_DELIMITER_CHAR,param3,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_send_2int(int op_code,int param1,int param2){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%d%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,param2,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_send_1int(int op_code,int param1){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_send(int op_code){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_send_2strint(int op_code,char* param1,int param2){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%s%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,param2,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_socket_send_2str(int op_code,char* param1,char* param2){
	#ifdef TS_RTAI
		rt_sem_wait(TSUSR_SEM_socket_lock);
	#else
		sem_wait(&TSUSR_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%s%c%s%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,param2,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSUSR_socket_send_client_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSUSR_trace(TSUSR_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSUSR_SEM_socket_lock);
	#else
		sem_post(&TSUSR_SEM_socket_lock);
	#endif
	return TSUSR_NORMAL_EXIT;
}
