/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr                                                      
** Fichier : usr_socket.h                                     
** Description : Gestion de la socket                             
------------------------------------------------------------------------------*/
/**
 * \file usr_socket.h
 * \brief Module : usr, gestion des sockets utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef USR_SOCKET_H
#define	USR_SOCKET_H
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include "usr_const_global.h"
#include "usr_const_socket.h"
#include "const_api.h"
#include "string_extra.h"

#ifdef TS_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif
/*
TSUSR_SEM
*/
#ifdef TS_RTAI
	static SEM* TSUSR_SEM_socket_lock;
#else
	static sem_t TSUSR_SEM_socket_lock;
#endif

/*
TSUSR_socket
*/
int TSUSR_socket_send_socket_id;
int TSUSR_socket_send_client_id;

/**
 * \fn int TSUSR_socket_load(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_load(void);
/**
 * \fn int TSUSR_socket_unload(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_unload(void);

/**
 * \fn int TSUSR_socket_createTcpClient(const char *hostname, int port)
 * \brief Creation d'un client TCP
 * \param hostname nom du serveur
 * \param port port
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_createTcpClient(const char *hostname, int port);
/**
 * \fn int TSUSR_socket_createTcpServer(int port, int nb_max_clients)
 * \brief Creation d'un serveur TCP
 * \param port port
 * \param nb_max_clients nombre de connexion maximal
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_createTcpServer(int port, int nb_max_clients);

/**
 * \fn int TSUSR_socket_read(char* recv_c)
 * \brief Lecture sur socket
 * \param recv_c caractere lu
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_read(char* recv_c);
/**
 * \fn int TSUSR_socket_readAnswer(void)
 * \brief Lecture de la reponse sur socket
 * \return reponse
 */
int TSUSR_socket_readAnswer(void);

/**
 * \fn int TSUSR_socket_send_3int(int op_code,int param1,int param2,int param3)
 * \brief Envoi sur socket d'un opcode et de 3 integer
 * \param op_code code operation
 * \param param1 parametre
 * \param param2 parametre
 * \param param3 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send_3int(int op_code,int param1,int param2,int param3);
/**
 * \fn int TSUSR_socket_send_2int(int op_code,int param1,int param2)
 * \brief Envoi sur socket d'un opcode et de 2 integer
 * \param op_code code operation
 * \param param1 parametre
 * \param param2 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send_2int(int op_code,int param1,int param2);
/**
 * \fn int TSUSR_socket_send_1int(int op_code,int param1)
 * \brief Envoi sur socket d'un opcode et d'un integer
 * \param op_code code operation
 * \param param1 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send_1int(int op_code,int param1);
/**
 * \fn int TSUSR_socket_send(int op_code)
 * \brief Envoi sur socket d'un opcode
 * \param op_code code operation
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send(int op_code);

/**
 * \fn int TSUSR_socket_send_2strint(int op_code,char* param1,int param2)
 * \brief Envoi sur socket d'un opcode, d'une chaine de caracteres et d'un integer
 * \param op_code code operation
 * \param param1 parametre
 * \param param2 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send_2strint(int op_code,char* param1,int param2);
/**
 * \fn int TSUSR_socket_send_2str(int op_code,char* param1,char* param2)
 * \brief Envoi sur socket d'un opcode, de 2 chaine de caracteres
 * \param op_code code operation
 * \param param1 parametre
 * \param param2 parametre
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_socket_send_2str(int op_code,char* param1,char* param2);
#endif
