/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr_tasks                                                      
** Fichier : usr_tasks.c                                    
** Description : Module usr_tasks                          
------------------------------------------------------------------------------*/
/**
 * \file usr_tasks.c
 * \brief Module : usr, gestion des taches utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "usr_tasks.h"
/*
TSUSR_tasks
*/
int TSUSR_tasks_sched_init(void){
	//Initialisation des semaphores
	#ifdef TSUSR_RTAI
		TSUSR_SEM_tasks_sched_mutex=rt_sem_init(TSUSR_SEM_tasks_sched_mutex,1);
		pthread_cond_init(&TSUSR_tasks_COND_tasks_res_monitor,0);
	#else
		sem_init(&TSUSR_SEM_tasks_sched_mutex,0,1);
		pthread_cond_init(&TSUSR_tasks_COND_tasks_res_monitor,0);
	#endif
	TSUSR_trace(TSUSR_TRACE_INFO,"tasks : scheduler init");
	return TSUSR_NORMAL_EXIT;
}

int TSUSR_tasks_sched_end(void){
	//Destruction des semaphores
	#ifdef TSUSR_RTAI
		rt_sem_delete(TSUSR_SEM_tasks_sched_mutex);
		pthread_cond_destroy(&TSUSR_tasks_COND_tasks_res_monitor);
	#else
		sem_destroy(&TSUSR_SEM_tasks_sched_mutex);
		pthread_cond_destroy(&TSUSR_tasks_COND_tasks_res_monitor);
	#endif
	TSUSR_trace(TSUSR_TRACE_INFO,"tasks : scheduler end");
	return TSUSR_NORMAL_EXIT;
}

void TSUSR_tasks_sched_request(int nb_cpu,char* threadName){
	#ifdef TSUSR_RTAI
		rt_sem_wait(TSUSR_SEM_tasks_sched_mutex);
	#else
		sem_wait(&TSUSR_SEM_tasks_sched_mutex);
	#endif
        while(TSUSR_tasks_available_res < nb_cpu){
		pthread_cond_wait(&TSUSR_tasks_COND_tasks_res_monitor,&TSUSR_SEM_tasks_sched_mutex);
	}
	TSUSR_tasks_available_res = TSUSR_tasks_available_res - nb_cpu;
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"tasks : scheduler allocates %d CPU for thread %s (%d CPU remaining)",nb_cpu,threadName,TSUSR_tasks_available_res);
	TSUSR_trace(TSUSR_TRACE_DEBUG,message);
	free(message);
	#ifdef TSUSR_RTAI
		rt_sem_signal(TSUSR_SEM_tasks_sched_mutex);
	#else
		sem_post(&TSUSR_SEM_tasks_sched_mutex);
	#endif
}

void TSUSR_tasks_sched_release(int nb_cpu,char* threadName){
	#ifdef TSUSR_RTAI
		rt_sem_wait(TSUSR_SEM_tasks_sched_mutex);
	#else
		sem_wait(&TSUSR_SEM_tasks_sched_mutex);
	#endif
	TSUSR_tasks_available_res = TSUSR_tasks_available_res + nb_cpu;
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"tasks : scheduler releases  %d CPU for thread %s (%d CPU remaining)",nb_cpu,threadName,TSUSR_tasks_available_res);
	TSUSR_trace(TSUSR_TRACE_DEBUG,message);
	free(message);
	pthread_cond_broadcast(&TSUSR_tasks_COND_tasks_res_monitor);
	#ifdef TSUSR_RTAI
		rt_sem_signal(TSUSR_SEM_tasks_sched_mutex);
	#else
		sem_post(&TSUSR_SEM_tasks_sched_mutex);
	#endif
	srand(time(NULL));
	usleep((TSUSR_TASKS_ATTENTE_MAX-rand())%TSUSR_TASKS_ATTENTE_MAX);
}

void* TSUSR_tasks_genericTask(int indx_arg,int rt_state,int cyclic_state,char* taskName,void (*taskInit)(void),void (*taskBody)(void),void (*taskEnd)(void)){
	#ifdef TSUSR_RTAI
		RTIME start_time, period;
		RTIME t0, t;
		RT_TASK *mytask;
		unsigned long mytask_name;
		int mytask_indx;

		mytask_indx = indx_arg;
		mytask_name = TSUSR_TASKS_taskname(mytask_indx);
		TSUSR_tasks_cpus_allowed = 1 - TSUSR_tasks_cpus_allowed; 
	 	if (!(mytask = rt_task_init_schmod(mytask_name, 1, 0, 0, SCHED_FIFO, 1 << TSUSR_tasks_cpus_allowed))) {
			char* message=(char*)malloc(1024*sizeof(char));
			sprintf(message,"tasks : cannot init %s task",taskName);
			TSUSR_trace(TSUSR_TRACE_ERROR,message);
			free(message);
			exit(TSUSR_ERROR_EXIT);
		}
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"tasks : start %s task with address = %p",taskName,mytask);
		TSUSR_trace(TSUSR_TRACE_DEBUG,message);
		free(message);
		mlockall(MCL_CURRENT | MCL_FUTURE);

	 	if (rt_state==TSUSR_TASKS_HARD_RT) {
			rt_make_hard_real_time();
		} else {
			rt_make_soft_real_time();
		}

		if (cyclic_state==TSUSR_TASKS_CYCLIC){
			period = nano2count(TSUSR_TASKS_PERIOD);
			start_time = rt_get_time() + nano2count(10000000);
			rt_task_make_periodic(mytask, start_time + (mytask_indx + 1)*period, TRAINS_NB_LOCOMOTIVES*period);
		}

	// start of task body
		{
	#else
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"tasks : start %s task",taskName);
		TSUSR_trace(TSUSR_TRACE_DEBUG,message);
		free(message);
	#endif

	if(taskInit!=NULL){
		taskInit();
	}

	if (cyclic_state==TSUSR_TASKS_ONESHOT || cyclic_state==TSUSR_TASKS_CYCLIC_AUTO){
		if(taskBody!=NULL){
			//TSUSR_tasks_sched_request(1,taskName);//thread_yield???
			taskBody();
			//TSUSR_tasks_sched_release(1,taskName);
		}
	} else if (cyclic_state==TSUSR_TASKS_CYCLIC){
		while (1){
			if(taskBody!=NULL){
				//TSUSR_tasks_sched_request(1,taskName);
				taskBody();
				//TSUSR_tasks_sched_release(1,taskName);
			}
		}
	}
	
	#ifdef TSUSR_RTAI
		}
	// end of task body
	 	if (rt_state==TSUSR_TASKS_HARD_RT) {
			rt_make_soft_real_time();
		}

		rt_task_delete(mytask);
	#endif
	if(taskEnd!=NULL) taskEnd();
	char* messageEnd=(char*)malloc(1024*sizeof(char));
	sprintf(messageEnd,"tasks : end %s task",taskName);	
	TSUSR_trace(TSUSR_TRACE_DEBUG,messageEnd);
	free(messageEnd);
	return 0;
}

/*_generic*/
void* TSUSR_tasks_generic(void* arg){
	TSUSR_tasks_params_s params=*((TSUSR_tasks_params_s*)arg);
	return	TSUSR_tasks_genericTask(params.mytask_indx,params.rt_state,params.cyclic_state,params.taskName,params.taskInit,params.taskBody,params.taskEnd);
}
