/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : usr_tasks                                                      
** Fichier : usr_tasks.h                                    
** Description : Module usr_tasks                          
------------------------------------------------------------------------------*/
/**
 * \file usr_tasks.h
 * \brief Module : usr, gestion des taches utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef USR_TASKS_H
#define USR_TASKS_H
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <pthread.h>
#include "usr.h"
#include "const_trains.h"
#include "usr_const_global.h"
#include "usr_const_tasks.h"
#ifdef TSUSR_RTAI
	#include <rtai_sem.h>
#else
	#include <semaphore.h>
#endif

static int TSUSR_trace_level=TSUSR_TRACE_NO;

/*
TSUSR_SEM
*/
#ifdef TSUSR_RTAI
	static SEM* TSUSR_SEM_tasks_sched_mutex;
	static pthread_cond_t TSUSR_tasks_COND_tasks_res_monitor;

	static SEM* TSUSR_SEM_tasks_params_mutex;
#else
	static sem_t TSUSR_SEM_tasks_sched_mutex;
	static pthread_cond_t TSUSR_tasks_COND_tasks_res_monitor;

	static sem_t TSUSR_SEM_tasks_params_mutex;
#endif

/*
TSUSR_tasks
*/
typedef struct TSUSR_tasks_params_s {
	int mytask_indx;
	int rt_state;
	int cyclic_state;
	char* taskName;
	void (*taskInit)(void);
	void (*taskBody)(void);
	void (*taskEnd)(void);
} TSUSR_tasks_params_s;


static int TSUSR_tasks_available_res=TSUSR_TASKS_NB_MAX_CPU;

static int TSUSR_tasks_argc;
static char** TSUSR_tasks_argv;

static pthread_t TSUSR_tasks[TRAINS_NB_LOCOMOTIVES];
static int TSUSR_tasks_indx[TRAINS_NB_LOCOMOTIVES];
static int TSUSR_tasks_cpus_allowed;

static TSUSR_tasks_params_s TSUSR_trains_params[TRAINS_NB_LOCOMOTIVES];

/**
 * \fn int TSUSR_tasks_params_init(void)
 * \brief Chargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_tasks_params_init(void);
/**
 * \fn int TSUSR_tasks_params_end(void)
 * \brief Dechargement du module
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_tasks_params_end(void);

/*
_sched
*/
/**
 * \fn int TSUSR_tasks_sched_init(void)
 * \brief Initilisation de l'ordonnanceur
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_tasks_sched_init(void);
/**
 * \fn int TSUSR_tasks_sched_end(void)
 * \brief Fin de l'ordonnanceur
 * \return int code etat (see const_kernel.h)
 */
int TSUSR_tasks_sched_end(void);
/**
 * \fn void TSUSR_tasks_sched_request(int nb_cpu,char* threadName)
 * \brief Alloacation du CPU par l'ordonnanceur
 * \param nb_cpu nombre de CPU
 * \param threadName nom de la tache
 * \return void
 */
void TSUSR_tasks_sched_request(int nb_cpu,char* threadName);
/**
 * \fn void TSUSR_tasks_sched_release(int nb_cpu,char* threadName)
 * \brief Desalloacation du CPU par l'ordonnanceur
 * \param nb_cpu nombre de CPU
 * \param threadName nom de la tache
 * \return void
 */
void TSUSR_tasks_sched_release(int nb_cpu,char* threadName);

/*_generic*/
/**
 * \fn void* TSUSR_tasks_genericTask(int indx_arg,int rt_state,int cyclic_state,char* taskName,void (*taskInit)(void),void (*taskBody)(void),void (*taskEnd)(void))
 * \brief Creation d'une tache generique
 * \param index_arg numero de tache
 * \param rt_state etat temps reel
 * \param taskName nom de la tache
 * \param taskInit pointeur sur fonction d'initilisation
 * \param taskBody pointeur sur fonction de corps
 * \param taskEnd pointeur sur fonction de fin
 * \return void* pointeur sur tache
 */
void* TSUSR_tasks_genericTask(int indx_arg,int rt_state,int cyclic_state,char* taskName,void (*taskInit)(void),void (*taskBody)(void),void (*taskEnd)(void));
/**
 * \fn void* TSUSR_tasks_generic(void* arg)
 * \brief Creation d'une tache generique
 * \param arg structure parametree de tache
 * \return void* pointeur sur tache
 */
void* TSUSR_tasks_generic(void* arg);

#endif
